/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.remoting.samples.oneway;

import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;

/**
 * Simple test client to make oneway invocations on remoting server.
 *
 * @author <a href="mailto:telrod@e2technologies.net">Tom Elrod</a>
 */
public class OnewayClient
{
   // Default locator values
   private static String transport = "socket";
   private static String host = "localhost";
   private static int port = 5400;

   public void makeInvocation(String locatorURI) throws Throwable
   {
      InvokerLocator locator = new InvokerLocator(locatorURI);
      System.out.println("Calling remoting server with locator uri of: " + locatorURI);

      Client remotingClient = new Client(locator);

      /**
       * Make oneway invocation.  The second parameter is for a map of payload
       * metadata that you would want to send.  Since this is a oneway invocation,
       * the return is void.
       *
       * With this invokeOneway signature, it uses the current thread to execute
       * the invocation on the server and will not return until the invocation request
       * has been placed into a worker thread pool on the server.  This ensures that
       * the call at least made it to the server.
       */
      String payload1 = "Oneway call 1.";
      System.out.println("Making oneway invocation with payload of '" + payload1 + "'");
      remotingClient.invokeOneway(payload1, null);

      /**
       * This call is the same as the one above, except the last parameter will
       * place the invocation request into a worker thread pool on the client and
       * return immediately.  The worker thread will then make the invocation
       * on the remoting server.
       */
      String payload2 = "Oneway call 2.";
      System.out.println("Making oneway invocation with payload of '" + payload2 + "'");
      remotingClient.invokeOneway(payload2, null, true);
   }

   /**
    * Can pass transport and port to be used as parameters.
    * Valid transports are 'rmi' and 'socket'.
    *
    * @param args
    */
   public static void main(String[] args)
   {
      if(args != null && args.length == 2)
      {
         transport = args[0];
         port = Integer.parseInt(args[1]);
      }
      String locatorURI = transport + "://" + host + ":" + port;
      OnewayClient client = new OnewayClient();
      try
      {
         client.makeInvocation(locatorURI);
      }
      catch(Throwable e)
      {
         e.printStackTrace();
      }
   }


}