/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/
package org.jboss.remoting;

import java.util.HashMap;
import java.util.Map;
import org.jboss.logging.Logger;
import org.jboss.remoting.loading.ClassByteClassLoader;
import org.jboss.remoting.marshal.MarshallLoaderFactory;

/**
 * AbstractInvoker is an abstract handler part that contains common methods between both
 * client and server.
 *
 * @author <a href="mailto:jhaynie@vocalocity.net">Jeff Haynie</a>
 * @author <a href="mailto:telrod@e2technologies.net">Tom Elrod</a>
 * @version $Revision: 1.4 $
 */
public abstract class AbstractInvoker implements Invoker
{
   protected final Logger log = Logger.getLogger(getClass());
   protected ClassByteClassLoader classbyteloader;
   protected InvokerLocator locator;
   protected Map localServerLocators = new HashMap();

   public AbstractInvoker(InvokerLocator locator)
   {
      this.classbyteloader = new ClassByteClassLoader(this.getClass().getClassLoader());
      this.locator = locator;

      try
      {
         InvokerLocator loaderLocator = MarshallLoaderFactory.convertLocator(locator);
         if(loaderLocator != null)
         {
            classbyteloader.setClientInvoker(new Client(loaderLocator));
         }
      }
      catch(Exception e)
      {
         log.error("Could not create remote class loading for invoker.", e);
      }
   }

   /**
    * return the locator this Invoker represents
    *
    * @return
    */
   public InvokerLocator getLocator()
   {
      return locator;
   }

   /**
    * Sets the callback server locator for the specified callback listener id
    *
    * @param locator
    */
   public void addClientLocator(String listenerId, InvokerLocator locator)
   {
      Object obj = localServerLocators.put(listenerId, locator);
      if(obj != null)
      {
         throw new RuntimeException("InvokerLocator already exists for listener id " + listenerId + ".  " +
                                    "Remove this listener before adding again.");
      }
   }

   /**
    * Gets the client locator.  This locator will be used by the server side
    * to make callbacks to the handler for this locator.
    *
    * @return
    */
   public InvokerLocator getClientLocator(String listenerId)
   {
      return (InvokerLocator) localServerLocators.get(listenerId);
   }

   /**
    * set the classloader to use internally
    *
    * @param classloader
    */
   public synchronized void setClassLoader(ClassLoader classloader)
   {
      this.classbyteloader = new ClassByteClassLoader(classloader);
   }

   public synchronized ClassLoader getClassLoader()
   {
      return classbyteloader;
   }

}
