/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.remoting.callback;

import java.util.Map;
import org.jboss.remoting.InvocationRequest;
import org.jboss.remoting.InvokerLocator;


/**
 * This is the class to use for sending callback payloads from the
 * server handler to the callback listener.
 *
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class Callback extends InvocationRequest
{
   static final long serialVersionUID = -4778964132014467531L;

   public final static String CALLBACK_HANDLE_OBJECT_KEY = "callback_handle_object";
   public final static String SERVER_LOCATOR_KEY = "server_locator";

   /**
    * Constructs the callback object with any object payload.
    *
    * @param callbackPayload
    */
   public Callback(Object callbackPayload)
   {
      super(callbackPayload);
   }

   /**
    * Returns the handle object originally specified when initially registering
    * the callback listener.  This can be used to provide context upon callbacks.
    *
    * @return
    */
   public Object getCallbackHandleObject()
   {
      Object handleObject = null;
      Map returnPayload = getReturnPayload();
      if(returnPayload != null)
      {
         handleObject = returnPayload.get(CALLBACK_HANDLE_OBJECT_KEY);
      }
      return handleObject;
   }

   /**
    * Gets the callback payload sent from the server handler.
    *
    * @return
    */
   public Object getCallbackObject()
   {
      return getParameter();
   }

   /**
    * Gets the locator for the target server where the callback was generated (this will be for
    * the same server that the callback client was registered).
    *
    * @return
    */
   public InvokerLocator getServerLocator()
   {
      InvokerLocator locator = null;
      Map returnPayload = getReturnPayload();
      if(returnPayload != null)
      {
         locator = (InvokerLocator) returnPayload.get(SERVER_LOCATOR_KEY);
      }
      return locator;
   }
}