/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/
package org.jboss.remoting.network;

import java.io.Serializable;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.detection.ServerInvokerMetadata;
import org.jboss.remoting.ident.Identity;

/**
 * NetworkInstance is an object that represents an Identity and its InvokerLocators.
 *
 * @author <a href="mailto:jhaynie@vocalocity.net">Jeff Haynie</a>
 * @version $Revision: 1.3 $
 */
public class NetworkInstance implements Serializable
{
   static final long serialVersionUID = -1745108606611832280L;

   private final Identity identity;
   private final ServerInvokerMetadata serverInvokers[];
   private final InvokerLocator locators[];
   private final int hashCode;

   public NetworkInstance(Identity identity, InvokerLocator locators[])
   {
      this.identity = identity;
      this.locators = locators;
      this.hashCode = this.identity.hashCode();
      this.serverInvokers = null;
   }

   public NetworkInstance(Identity identity, ServerInvokerMetadata serverInvokers[])
   {
      this.identity = identity;
      this.locators = null;
      this.hashCode = this.identity.hashCode();
      this.serverInvokers = serverInvokers;
   }

   public final Identity getIdentity()
   {
      return identity;
   }

   public final InvokerLocator[] getLocators()
   {
      if(locators != null)
      {
         return locators;
      }
      else
      {
         InvokerLocator[] locators = new InvokerLocator[serverInvokers.length];
         for(int x = 0; x < serverInvokers.length; x++)
         {
            locators[x] = serverInvokers[x].getInvokerLocator();
         }
         return locators;
      }

   }

   /**
    * Gets all the server invoker metadata for the servers running on
    * specified server.  Will include locator and supported subsystems
    * for that locator.
    *
    * @return
    */
   public final ServerInvokerMetadata[] getServerInvokers()
   {
      return serverInvokers;
   }


   public String toString()
   {
      return "NetworkInstance [identity:" + identity + ",locator count:" + (serverInvokers == null ? 0 : serverInvokers.length) + "]";
   }

   public boolean equals(Object obj)
   {
      return hashCode == obj.hashCode();
   }

   public int hashCode()
   {
      return hashCode;
   }
}
