/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.remoting.samples.stream;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;

/**
 * This samples shows how the client can send an InputStream to
 * the server.
 *
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class StreamingClient
{
   // Default locator values
   private static String transport = "socket";
   private static String host = "localhost";
   private static int port = 5400;

   private String locatorURI = transport + "://" + host + ":" + port;

   private String localFileName = "sample.txt";
   private String remoteFileName = "server_sample.txt";

   public void sendStream() throws Throwable
   {
      FileInputStream fileInput = null;
      Client remotingClient = null;

      try
      {
         InvokerLocator locator = new InvokerLocator(locatorURI);
         System.out.println("Calling on remoting server with locator uri of: " + locatorURI);

         remotingClient = new Client(locator);
         URL fileURL = this.getClass().getResource(localFileName);
         if(fileURL == null)
         {
            throw new Exception("Can not find file " + localFileName);
         }
         File testFile = new File(fileURL.getFile());
         fileInput = new FileInputStream(testFile);

         System.out.println("Sending input stream for file " + localFileName + " to server.");
         Object ret = remotingClient.invoke(fileInput, remoteFileName);

         long fileLength = testFile.length();
         System.out.println("Size of file sample.txt is " + fileLength);
         System.out.println("Server returned " + ret + " as the size of the file read.");

      }
      finally
      {
         if(remotingClient != null)
         {
            remotingClient.disconnect();
         }
         if(fileInput != null)
         {
            try
            {
               fileInput.close();
            }
            catch(IOException e)
            {
               e.printStackTrace();
            }
         }
      }
   }

   public void setRemoteFileName(String remoteFileName)
   {
      this.remoteFileName = remoteFileName;
   }


   /**
    * Can pass transport and port to be used as parameters.
    * Valid transports are 'rmi' and 'socket'.
    *
    * @param args
    */
   public static void main(String[] args)
   {
      String newFileName = null;
      if(args != null && args.length == 1)
      {
         newFileName = args[0];
      }
      if(args != null && args.length == 2)
      {
         transport = args[0];
         port = Integer.parseInt(args[1]);
      }
      if(args != null && args.length == 3)
      {
         transport = args[0];
         port = Integer.parseInt(args[1]);
         newFileName = args[2];
      }
      String locatorURI = transport + "://" + host + ":" + port;
      StreamingClient client = new StreamingClient();
      if(newFileName != null)
      {
         client.setRemoteFileName(newFileName);
      }
      try
      {
         client.sendStream();
      }
      catch(Throwable e)
      {
         e.printStackTrace();
      }
   }


}