/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.remoting.stream;

import java.io.Serializable;

/**
 * The StreamCallPayload is used when making calls from the server
 * to the client to read from the original input stream.
 * It will contain the method name being called from the server side (i.e. available(), read(), etc.)
 * along with any parameters for the respective method.
 *
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class StreamCallPayload implements Serializable
{
   static final long serialVersionUID = 3243358524048714566L;
   
   private String method = null;
   private Object[] paramArray = null;

   /**
    * Constructor which requires the name of the method to call on the
    * the original stream.
    *
    * @param methodCallName
    */
   public StreamCallPayload(String methodCallName)
   {
      this.method = methodCallName;
   }

   /**
    * Gets the method to call on the original stream.
    *
    * @return
    */
   public String getMethod()
   {
      return method;
   }

   /**
    * Sets the params for the method to call on the
    * stream.  For example, the Integer for markSupported(int supported).
    *
    * @param params
    */
   public void setParams(Object[] params)
   {
      this.paramArray = params;
   }

   /**
    * Gets the params for the method to call on the stream.
    *
    * @return
    */
   public Object[] getParams()
   {
      return this.paramArray;
   }
}