/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.remoting.stream;

import java.io.InputStream;
import org.jboss.remoting.ServerInvocationHandler;

/**
 * This interface is intended for those handlers that expect to
 * receive calls from clients where a InputStream will be passed.
 *
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public interface StreamInvocationHandler extends ServerInvocationHandler
{
   /**
    * Will receive an input stream, which is actually a proxy back
    * to the original stream on the client's vm.  This stream can
    * be acted upon as though it was a local input stream.
    * When finished reading from the stream, it MUST be closed, or
    * will remain open on the client side.
    *
    * @param stream stream proxy to client's original stream
    * @param param  the payload associated with the invocation
    */
   public Object handleStream(InputStream stream, Object param);
}