/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.remoting.transport.http.ssl;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.util.Map;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.transport.http.HTTPClientInvoker;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class HTTPSClientInvoker extends HTTPClientInvoker
{
   /**
    * A property to override the default https url host verification
    */
   public static final String IGNORE_HTTPS_HOST = "org.jboss.security.ignoreHttpsHost";

   public HTTPSClientInvoker(InvokerLocator locator)
   {
      super(locator);
   }

   /**
    * Checks to see if org.jboss.security.ignoreHttpHost property is set to true, and if it
    * is, will ste the host name verifier so that will accept any host.
    *
    * @return
    * @throws IOException
    */
   protected HttpURLConnection createURLConnection(String url, Map metadat) throws IOException
   {
      HttpURLConnection conn = super.createURLConnection(url, metadat);

      if(conn instanceof HttpsURLConnection && Boolean.getBoolean(IGNORE_HTTPS_HOST) == true)
      {
         HttpsURLConnection sconn = (HttpsURLConnection) conn;
         sconn.setHostnameVerifier(new AnyhostVerifier());
      }
      return conn;
   }

   private class AnyhostVerifier implements HostnameVerifier
   {

      public boolean verify(String s, SSLSession sslSession)
      {
         return true;
      }
   }
}