/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 *
 * Created on Jul 24, 2005
 */
package org.jboss.remoting.transport.multiplex;

import java.io.DataInputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

import org.jboss.logging.Logger;


/**
 * 
 * <p>
 * Copyright (c) 2005
 * <p>
 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 */
public class MultiPortVirtualServerSocket extends ServerSocket
{
   private static final Logger log = Logger.getLogger(MultiPortVirtualServerSocket.class);

   
//////////////////////////////////////////////////////////////////////////////////////////////////
///        The following constructors duplicate those of the parent class ServerSocket        '///
//////////////////////////////////////////////////////////////////////////////////////////////////
   
/**
 * @throws java.io.IOException
 */
   public MultiPortVirtualServerSocket() throws IOException
   {
      super();
   }

/**
 * @param port
 * @throws java.io.IOException
 */
   public MultiPortVirtualServerSocket(int port) throws IOException
   {
      super(port);
   }

/**
 * @param port
 * @param backlog
 * @throws java.io.IOException
 */
   public MultiPortVirtualServerSocket(int port, int backlog) throws IOException
   {
      super(port, backlog);
   }

/**
 * @param port
 * @param backlog
 * @param bindAddr
 * @throws java.io.IOException
 */
   public MultiPortVirtualServerSocket(int port, int backlog, InetAddress bindAddr) throws IOException
   {
      super(port, backlog, bindAddr);
   }
   
   
//////////////////////////////////////////////////////////////////////////////////////////////////
///                  The following methods are required of any ServerSocket                   '///
//////////////////////////////////////////////////////////////////////////////////////////////////

   
   /*
     ok: public void bind(SocketAddress endpoint) throws IOException;
     ok: public void bind(SocketAddress endpoint, int backlog) throws IOException;
     ok: public InetAddress getInetAddress();
     ok: public int getLocalPort();
     ok: public SocketAddress getLocalSocketAddress();
     ok: public Socket accept() throws IOException;
     ok: public void close() throws IOException;
     ok: public ServerSocketChannel getChannel();
     ok: public boolean isBound();
     ok: public boolean isClosed();
     ok: public void setSoTimeout(int timeout) throws SocketException;
     ok: public int getSoTimeout() throws IOException;
     ok: public void setReuseAddress(boolean on) throws SocketException;
     ok: public boolean getReuseAddress() throws SocketException;
     ok: public String toString();
     ok: public void setReceiveBufferSize(int size) throws SocketException;
     ok: public int getReceiveBufferSize() throws SocketException;
   */
   
/**
 * 
 */
   public Socket accept() throws IOException
   {
      while (true)
      {
         Socket socket = super.accept();
         DataInputStream is = new DataInputStream(socket.getInputStream());
         SocketId clientPort = Protocol.acceptConnect(is);
         log.debug("accept(): clientPort:  " + clientPort.getPort());
         MultiplexingManager manager = MultiplexingManager.getaManager(socket);
         Protocol protocol = manager.getProtocol();
         
         // connection from independent SinglePortVirtualServerSocket
         if (clientPort.getPort() < 0)
         {
            protocol.answerConnect(SocketId.SERVER_SOCKET_PORT);
            continue;
         }

         VirtualSocket virtualSocket = new VirtualSocket(manager, clientPort);
         int localPort = virtualSocket.getLocalVirtualPort();
         protocol.answerConnect(localPort);
         return virtualSocket;
      }
   }
   
   
/**
 * 
 */
   public void close() throws IOException
   {
      log.info("MultiPortVirtualServerSocket: closing");
      super.close();
   }

   
/**
 * 
 */
   public String toString()
   {
  	if (!isBound())
  	    return "MultiPortVirtualServerSocket[unbound]";
  	
  	return "MultiPortVirtualServerSocket[addr=" + getInetAddress() +  ",localport=" + getLocalPort()  + "]";
   }
   
   
//////////////////////////////////////////////////////////////////////////////////////////////////
///              The following methods are specific to MultiPortVirtualServerSocket           '///
//////////////////////////////////////////////////////////////////////////////////////////////////
}
