/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 *
 * Created on Jul 22, 2005
 */
package org.jboss.remoting.transport.multiplex;

import java.io.IOException;
import java.io.OutputStream;
import java.net.SocketException;

import org.jboss.logging.Logger;

/**
 * <p>
 * Copyright (c) 2005
 * <p>
 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 */
public class MultiplexingOutputStream extends OutputStream
{
   protected static final Logger log = Logger.getLogger(MultiplexingOutputStream.class);
   private MultiplexingManager manager;
   private OutputMultiplexor outputMultiplexor;
   private VirtualSocket virtualSocket;
   private SocketId socketId;
   private boolean shutdown = false;
   private boolean closed = false;
   private boolean remoteShutDownPending = false;
   private boolean remoteOutputShutdown = false;

   
   
/**
 * 
 * @param manager
 * @param socketId
 */
   public MultiplexingOutputStream(MultiplexingManager manager, SocketId socketId)
   {
      this(manager, null, socketId);
   }
   
   
/**
 * 
 * @param manager
 * @param virtualSocket TODO
 * @param socketId
 */
   public MultiplexingOutputStream(MultiplexingManager manager, VirtualSocket virtualSocket, SocketId socketId)
   {
      this.manager = manager;
      this.virtualSocket = virtualSocket;
      this.socketId = socketId;
      this.outputMultiplexor = manager.getOutputMultiplexor();
   }
   
   
/**
 * 
 */
   public void write(int i) throws IOException
   {
      checkStatus();
      outputMultiplexor.write(socketId, new byte[] {(byte) i});
   }
   
   
   public void write(byte[] array) throws IOException, NullPointerException
   { 
      checkStatus();
      outputMultiplexor.write(socketId, array);
   }
   
   
   public void write(byte[] array, int off, int len)
   	throws IOException, NullPointerException, IndexOutOfBoundsException
   { 
      checkStatus();
      
      if (array == null)
         throw new NullPointerException();
      
      if (off < 0 || len < 0 || off + len > array.length)
         throw new IndexOutOfBoundsException();
      
      byte[] subArray = new byte[len];
      
      for (int i = 0; i < len; i++)
         subArray[i] = array[off + i];

      outputMultiplexor.write(socketId, subArray);
   }
   
   
   public void close() throws IOException
   {
      if (closed)
         return;
         
      closed = true;
      
      if (virtualSocket != null)
         virtualSocket.close();
   }
   
   
/**
 *  FIXME Comment this
 * 
 * @throws IOException
 */
   protected void checkStatus() throws IOException
   {
      if (closed)
         throw new SocketException("Socket closed");
      
      if (shutdown)
         throw new SocketException("Broken pipe");
   }
   
   
/**
 *  FIXME Comment this
 * 
 *
 */
   protected void handleRemoteInputShutdown()
   {
      log.debug("entering handleRemoteInputShutdown()");
      shutdown = true;
   }
   
   
/**
 *  FIXME Comment this
 * 
 *
 */
   protected void shutdown()
   {
      shutdown = true;
   }
}
