/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 *
 * Created on Jul 22, 2005
 */
package org.jboss.remoting.transport.multiplex;

import java.io.IOException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.HashSet;
import java.util.Iterator;

import org.apache.log4j.Logger;

/**
 * <p>
 * Copyright (c) 2005
 * <p>
 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 * 
 * TODO: verify bytes has no 0's
 */
public class SocketId implements Serializable
{
   protected static final Logger log = Logger.getLogger(SocketId.class);
   private static final String ISO_8859_1 = "ISO-8859-1";
   
   public static final int PROTOCOL_PORT 		= -1;
   public static final int SERVER_SOCKET_PORT 	= -2;
   public static final int BACKCHANNEL_PORT 	= -3;
   public static final int DEADLETTER_PORT 		= -4;
   public static final int INITIAL_OUTPUT_PORT 	= -5;
   private static final int MIN_PORT			=  INITIAL_OUTPUT_PORT;

   public static final SocketId PROTOCOL_SOCKET_ID 	 = new SocketId((short) PROTOCOL_PORT);
   public static final SocketId SERVER_SOCKET_ID 		 = new SocketId((short) SERVER_SOCKET_PORT);
   public static final SocketId BACKCHANNEL_SOCKET_ID 	 = new SocketId((short) BACKCHANNEL_PORT);
   public static final SocketId DEADLETTER_SOCKET_ID 	 = new SocketId((short) DEADLETTER_PORT);
   public static final SocketId INITIAL_OUTPUT_SOCKET_ID = new SocketId((short) INITIAL_OUTPUT_PORT);

   private static HashSet portsInUse = new HashSet();
   private static HashSet portsFreed = new HashSet();
   private static int nextNewPort = 1;
   
   private int port;
   byte[] bytes;
   
/**
 *
 */
   public SocketId() throws IOException
   {
      port = getFreePort();
      setBytes(port);
   }
   
   
/**
 * 
 * @param port
 */
   public SocketId(int port) throws IOException
   {
      checkPortValue(port);
      this.port = port;
      setBytes(port);
   } 
   
   
/**
 * 
 * @param id
 */
   public SocketId(byte[] bytes) throws IOException
   {
      this.bytes = bytes;
      setPort(bytes);
      checkPortValue(port);
   }
   

/**
 * 
 * @param port
 */
   protected SocketId(short port)
   {
      this.port = port;
      setBytes(port);
   }
   
   
   public byte[] toByteArray()
   {
      return bytes;
   }
   
   
   public int getPort()
   {
      return port;
   }

   
   public boolean equals(Object o)
   {
      if (! (o instanceof SocketId))
         return false;
      
      SocketId other = (SocketId) o;
      
      if (this.port == other.port)
         return true;
      else
         return false;
   }
   
   
   public int hashCode()
   {
      return port;
   }
   
   
   protected void checkPortValue(int port) throws IOException
   {
      if (port < MIN_PORT)
      {
         log.error("attempt to create port with illegal value: " + port);
         throw new IOException("attempt to create port with illegal value: " + port);
      }
   }
   
   
   protected void setPort(byte[] bytes)
   {
      try
      {
         port = Integer.parseInt(new String(bytes, ISO_8859_1));
      }
      
      catch (UnsupportedEncodingException e)
      { 
         // ISO-8859-1 should be standard in all Java implementations
         log.fatal("org.jboss.remoting.transport.multiplex.OutputMultiplexor(): charset ISO-8859-1 not found", e);
         e.printStackTrace();
         throw new RuntimeException("org.jboss.remoting.transport.multiplex.OutputMultiplexor(): charset ISO-8859-1 not found", e);
      }
   }
   
   
   protected void setBytes(int port)
   {  
      try
      {
         bytes = Integer.toString(port).getBytes(ISO_8859_1);
      }
//      catch (NumberFormatException e)
//      {
//         String errorMessage = "SocketId(): port bytes do not represent a number";
//         log.error(errorMessage, e);
//         throw e;
//      }
      catch (UnsupportedEncodingException e)
      { 
         // ISO-8859-1 should be standard in all Java implementations
         log.fatal("org.jboss.remoting.transport.multiplex.OutputMultiplexor(): charset ISO-8859-1 not found", e);
         e.printStackTrace();
         throw new RuntimeException("org.jboss.remoting.transport.multiplex.OutputMultiplexor(): charset ISO-8859-1 not found", e);
      }
   }
   
   
   protected synchronized int getFreePort()
   {
      // reuse a previously used, then freed, port number
      if (!portsFreed.isEmpty())
      {
         Iterator it = portsFreed.iterator();
         Integer portInteger = (Integer) it.next();
         portsFreed.remove(portInteger);
         portsInUse.add(portInteger);  
         return portInteger.intValue();
      }
      
      // use a new port number
      int port = nextNewPort++;
      portsInUse.add(new Integer(port));
      return port;
   }
}
