/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.remoting.transport.socket;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;
import org.jboss.remoting.loading.ObjectInputStreamWithClassLoader;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class ClientSocketWrapper extends SocketWrapper
{
   private ObjectInputStream in;
   private ObjectOutputStream out;

   public ClientSocketWrapper(Socket socket) throws Exception
   {
      super(socket);
   }

   protected ObjectInputStream createInputStream(Socket socket)
         throws IOException
   {
      BufferedInputStream bin = new BufferedInputStream(socket.getInputStream());
      //oin = new ObjectInputStream(bin);
      ObjectInputStream oin = new ObjectInputStreamWithClassLoader(bin, null);
      return oin;
   }

   protected ObjectOutputStream createOutputStream(Socket socket)
         throws IOException
   {
      BufferedOutputStream bout = new BufferedOutputStream(socket.getOutputStream());
      ObjectOutputStream oout = new ObjectOutputStream(bout);
      return oout;
   }

   public OutputStream getOutputStream() throws IOException
   {
      if(out == null)
      {
         out = createOutputStream(getSocket());
      }
      return out;
   }

   public InputStream getInputStream() throws IOException
   {
      if(in == null)
      {
         in = createInputStream(getSocket());
      }
      return in;
   }

   public void checkConnection() throws IOException
   {
      // Test to see if socket is alive by send ACK message
      final byte ACK = 1;

      out.reset();
      out.writeByte(ACK);
      out.flush();
      in.readByte();

   }
}

