/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.remoting.transport.socket;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import org.jboss.logging.Logger;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class ServerSocketWrapper extends ClientSocketWrapper
{
   final static private Logger log = Logger.getLogger(ServerSocketWrapper.class);

   public ServerSocketWrapper(Socket socket) throws Exception
   {
      super(socket);
   }

   protected ObjectInputStream createInputStream(Socket socket) throws IOException
   {
      BufferedInputStream bin = new BufferedInputStream(socket.getInputStream());
      ObjectInputStream oin = new ObjectInputStream(bin);
      return oin;
   }

   protected ObjectOutputStream createOutputStream(Socket socket)
         throws IOException
   {
      BufferedOutputStream bout = new BufferedOutputStream(socket.getOutputStream());
      ObjectOutputStream oout = new ObjectOutputStream(bout);
      oout.flush();
      return oout;
   }

   public void checkConnection() throws IOException
   {
      // Perform acknowledgement to convince client
      // that the socket is still active
      byte ACK = 0;
      long startWait = System.currentTimeMillis();
      try
      {
         ACK = ((ObjectInputStream) getInputStream()).readByte();
      }
      catch(EOFException eof)
      {
         if(log.isTraceEnabled())
         {
            log.trace("socket timeout is set to : " + getTimeout());
            log.trace("EOFException waiting on ACK in readByte(). Time waited was " + (System.currentTimeMillis() - startWait));
         }
         throw eof;
      }
      catch(IOException e)
      {
         log.trace("IOException when reading in ACK", e);
         throw e;
      }

      if(log.isTraceEnabled())
      {
         log.trace("***acknowledge read byte" + Thread.currentThread());
      }

      ObjectOutputStream out = (ObjectOutputStream) getOutputStream();
      out.writeByte(ACK);
      out.flush();
      out.reset();
   }
}

