/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.remoting.transport.socket;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketException;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public abstract class SocketWrapper
{
   private Socket socket;
   private int timeout;

   public SocketWrapper(Socket socket)
   {
      this.socket = socket;
   }

   public abstract OutputStream getOutputStream() throws IOException;

   public abstract InputStream getInputStream() throws IOException;

   public abstract void checkConnection() throws IOException;

   public void setTimeout(int timeout) throws SocketException
   {
      this.timeout = timeout;
      if(socket != null)
      {
         socket.setSoTimeout(timeout);
      }
   }

   public int getTimeout()
   {
      return timeout;
   }


   public void close() throws IOException
   {
      if(socket != null)
      {
         socket.close();
      }
   }

   protected Socket getSocket()
   {
      return socket;
   }

   protected void finalize()
   {
      if(socket != null)
      {
         try
         {
            socket.close();
         }
         catch(Exception ignored)
         {
         }
      }
   }

}