/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting;

import java.io.IOException;
import java.net.ServerSocket;
import java.security.SecureRandom;
import org.jboss.logging.Logger;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class TestUtil
{
   private static final Logger log = Logger.getLogger(TestUtil.class);

   public static int getRandomPort()
   {
      Integer port = null;
      while(port == null)
      {
         port = getFreePort();
      }
      return port.intValue();
   }

   private static Integer getFreePort()
   {
      Integer port = null;
      ServerSocket socket = null;

      int p = Math.abs(new SecureRandom(String.valueOf(System.currentTimeMillis()).getBytes()).nextInt(2000));

      try
      {
         socket = new ServerSocket(p);
         port = new Integer(p);
      }
      catch(IOException e)
      {
         log.debug("port " + p + " already in use.  Will try another.");
      }
      finally
      {
         if(socket != null)
         {
            try
            {
               socket.close();
            }
            catch(IOException e)
            {

            }
         }
      }
      return port;
   }
}