/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.configuration;

import java.io.ByteArrayInputStream;
import java.net.InetAddress;
import javax.xml.parsers.DocumentBuilderFactory;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.InvokerRegistry;
import org.jboss.remoting.ServerInvoker;
import org.jboss.remoting.transport.Connector;
import org.jboss.remoting.transport.PortUtil;
import org.jboss.test.remoting.transport.mock.MockServerInvocationHandler;
import org.w3c.dom.Document;

import junit.framework.TestCase;


/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class RMIConfigurationTestCase extends TestCase
{
   private String transport = "rmi";
   private int serverPort = 6666;
   private int clientPort = 7777;
   private int registryPort = 8888;
   private String hostName = null;
   private String hostIP = null;
   private Connector connector = null;

   public void setUp() throws Exception
   {
      hostName = InetAddress.getLocalHost().getHostName();
      hostIP = InetAddress.getLocalHost().getHostAddress();

      connector = new Connector();
      StringBuffer buf = new StringBuffer();
      buf.append("<?xml version=\"1.0\"?>\n");
      buf.append("<config>");
      buf.append("<invoker transport=\"" + transport + "\">");
      buf.append("<attribute name=\"registryPort\" isParam=\"true\">" + registryPort + "</attribute>");
      buf.append("<attribute name=\"serverBindAddress\">" + hostName + "</attribute>");
      buf.append("<attribute name=\"serverBindPort\">" + serverPort + "</attribute>");
      buf.append("<attribute name=\"clientConnectAddress\">" + hostIP + "</attribute>");
      buf.append("<attribute name=\"clientConnectPort\">" + clientPort + "</attribute>");
      buf.append("</invoker>");
      buf.append("<handlers>");
      buf.append("  <handler subsystem=\"mock\">" + MockServerInvocationHandler.class.getName() + "</handler>\n");
      buf.append("</handlers>");
      buf.append("</config>");
      Document xml = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new ByteArrayInputStream(buf.toString().getBytes()));
      connector.setConfiguration(xml.getDocumentElement());
      connector.create();
      connector.start();

   }

   public void testClientConfiguration() throws Exception
   {

      // make sure the client's view of the locator will be as configured
      ServerInvoker[] serverInvokers = InvokerRegistry.getServerInvokers();

      if(serverInvokers != null && serverInvokers.length > 0)
      {
         InvokerLocator locator = serverInvokers[0].getLocator();
         String locatorHost = locator.getHost();
         int locatorPort = locator.getPort();

         System.out.println("locator host = " + locatorHost);
         System.out.println("locator port = " + locatorPort);
         assertEquals(hostIP, locatorHost);
         assertEquals(clientPort, locatorPort);
      }

      // check for server bind port (assume is the server since no exception thrown before)
      boolean portInUse = PortUtil.checkPort(serverPort);
      assertTrue(!portInUse);
      portInUse = PortUtil.checkPort(registryPort);
      assertTrue(!portInUse);

      // make sure can call ont
      Client client = new Client(new InvokerLocator(transport + "://" + hostName + ":" + serverPort + "/?registryPort=" + registryPort));
      String param = "foobar";
      Object ret = null;
      try
      {
         ret = client.invoke(param);
      }
      catch(Throwable throwable)
      {
         throw new Exception("Call on server failed.", throwable);
      }
      assertEquals(param, ret);

   }

   public void tearDown() throws Exception
   {
      if(connector != null)
      {
         connector.stop();
         connector.destroy();
      }
   }

   public static void main(String[] args)
   {
      RMIConfigurationTestCase testCase = new RMIConfigurationTestCase();
      try
      {
         testCase.setUp();
         testCase.testClientConfiguration();
         testCase.tearDown();
      }
      catch(Exception e)
      {
         e.printStackTrace();
      }
   }
}