/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.configuration.threadpool;

import java.io.ByteArrayInputStream;
import java.net.InetAddress;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.xml.parsers.DocumentBuilderFactory;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.ServerInvoker;
import org.jboss.remoting.transport.Connector;
import org.jboss.remoting.transport.http.HTTPServerInvoker;
import org.jboss.test.remoting.transport.mock.MockServerInvocationHandler;
import org.jboss.util.threadpool.BasicThreadPool;
import org.jboss.util.threadpool.ThreadPool;
import org.w3c.dom.Document;

import junit.framework.TestCase;


/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class HTTPThreadPoolConfigurationTestCase extends TestCase
{
   private String transport = "http";
   private int serverPort = 6666;
   private String hostName = null;
   private Connector connector = null;
   private BasicThreadPool tp = null;

   public void setUp() throws Exception
   {
      MBeanServer server = MBeanServerFactory.createMBeanServer();

      tp = new BasicThreadPool();
      ObjectName objName = new ObjectName("test:type=threadpool");
      server.registerMBean(tp, objName);


      hostName = InetAddress.getLocalHost().getHostName();

      connector = new Connector();
      StringBuffer buf = new StringBuffer();
      buf.append("<?xml version=\"1.0\"?>\n");
      buf.append("<config>");
      buf.append("<invoker transport=\"" + transport + "\">");
      buf.append("<attribute name=\"serverBindAddress\">" + hostName + "</attribute>");
      buf.append("<attribute name=\"serverBindPort\">" + serverPort + "</attribute>");
      //buf.append("<attribute name=\"" + HTTPServerInvoker.HTTP_THREAD_POOL_CLASS_KEY + "\">" + BasicThreadPool.class.getName() + "</attribute>");
      buf.append("<attribute name=\"" + HTTPServerInvoker.HTTP_THREAD_POOL_CLASS_KEY + "\">" + objName + "</attribute>");
      buf.append("</invoker>");
      buf.append("<handlers>");
      buf.append("  <handler subsystem=\"mock\">" + MockServerInvocationHandler.class.getName() + "</handler>\n");
      buf.append("</handlers>");
      buf.append("</config>");
      Document xml = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new ByteArrayInputStream(buf.toString().getBytes()));
      //connector.setInvokerLocator(locator.getLocatorURI());
      connector.setConfiguration(xml.getDocumentElement());
      server.registerMBean(connector, new ObjectName("test:type=connector,transport=http"));
      connector.create();
      connector.start();
   }

   public void testClientConfiguration() throws Exception
   {
      // make sure can call ont
      Client client = new Client(new InvokerLocator(transport + "://" + hostName + ":" + serverPort));
      String param = "foobar";
      Object ret = null;
      try
      {
         ret = client.invoke(param);
      }
      catch(Throwable throwable)
      {
         throwable.printStackTrace();
      }
      assertEquals(param, ret);

      ServerInvoker invoker = connector.getServerInvoker();
      HTTPServerInvoker httpInvoker = (HTTPServerInvoker)invoker;
      ThreadPool tp2 = httpInvoker.getHTTPThreadPool();
      assertEquals(tp, tp2);

   }

   public void tearDown() throws Exception
   {
      if(connector != null)
      {
         connector.stop();
         connector.destroy();
      }
   }

   public static void main(String[] args)
   {
      HTTPThreadPoolConfigurationTestCase testCase = new HTTPThreadPoolConfigurationTestCase();
      try
      {
         testCase.setUp();
         testCase.testClientConfiguration();
         testCase.tearDown();
      }
      catch(Exception e)
      {
         e.printStackTrace();
      }
   }
}