/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.connection;

import org.jboss.remoting.Client;
import org.jboss.remoting.ConnectionListener;
import org.jboss.remoting.InvokerLocator;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class ConnectionValidatorClient extends TestCase implements ConnectionListener
{
   // Default locator values
   private static String transport = "socket";
   private static String host = "localhost";
   private static int port = 5400;
   String locatorURI = transport + "://" + host + ":" + port;
   private Throwable validatorResp = null;

   private Client remotingClient = null;

   public void testValidator() throws Throwable
   {

      remotingClient.addConnectionListener(this);

      Object response = remotingClient.invoke("Do something");

      System.out.println("Invocation response: " + response);

      Thread.currentThread().sleep(30000);

      System.out.println("validatorResp = " + validatorResp);
      assertNotNull("Connection listener was not called as expected.", validatorResp);
   }

   public void setUp() throws Exception
   {
      InvokerLocator locator = new InvokerLocator(locatorURI);
      System.out.println("Calling remoting server with locator uri of: " + locatorURI);

      remotingClient = new Client(locator);
   }

   public void tearDown() throws Exception
   {
      if(remotingClient != null)
      {
         remotingClient.disconnect();
      }
   }

   /**
    * Can pass transport and port to be used as parameters.
    * Valid transports are 'rmi' and 'socket'.
    *
    * @param args
    */
   public static void main(String[] args)
   {
      if(args != null && args.length == 3)
      {
         transport = args[0];
         host = args[1];
         port = Integer.parseInt(args[2]);
      }

      ConnectionValidatorClient client = new ConnectionValidatorClient();
      try
      {
         client.setUp();
         client.testValidator();
         client.tearDown();
      }
      catch(Throwable e)
      {
         e.printStackTrace();
      }
   }

   public void handlerConnectionException(Throwable throwable, Client client)
   {
      System.out.println("Got connection exception.");
      validatorResp = throwable;
   }
}