/**
 * JBoss, the OpenSource J2EE WebOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.invoker;

import org.jboss.logging.Logger;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.InvokerRegistry;
import org.jboss.remoting.RemoteClientInvoker;
import org.jboss.remoting.transport.ClientInvoker;
import org.jboss.remoting.transport.Connector;
import org.jboss.remoting.transport.local.LocalClientInvoker;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:ovidiu@jboss.org">Ovidiu Feodorov</a>
 *
 * @version <tt>$Revision: 1.2 $</tt>
 *
 * $Id: ClientInvokerDisconnectTestCase.java,v 1.2 2005/06/20 03:38:43 telrod Exp $
 */
public class ClientInvokerDisconnectTestCase extends TestCase
{
   // Constants -----------------------------------------------------

   private static final Logger log = Logger.getLogger(ClientInvokerDisconnectTestCase.class);

   // Static --------------------------------------------------------

   // Attributes ----------------------------------------------------

   private String serverURI = "socket://127.0.0.1:5555/";
   private Connector server;

   // Constructors --------------------------------------------------

   public ClientInvokerDisconnectTestCase(String name)
   {
      super(name);
   }

   // Public --------------------------------------------------------

   public void setUp() throws Exception
   {
      super.setUp();

      server = new Connector();
      server.setInvokerLocator(serverURI);
      server.start();
      log.info("Server " + serverURI + " started");
   }

   public void tearDown() throws Exception
   {
      server.stop();
      server = null;
      log.info("Server " + serverURI + " stopped");

      super.tearDown();
   }

   public void testLocalClientInvokerDisconnect() throws Throwable
   {
      Client client = new Client(new InvokerLocator(serverURI));

      ClientInvoker[] clientInvokers = InvokerRegistry.getClientInvokers();
      assertEquals(1, clientInvokers.length);

      LocalClientInvoker clientInvoker = (LocalClientInvoker)clientInvokers[0];
      assertEquals(serverURI, clientInvoker.getLocator().getLocatorURI());


      client.disconnect();


      clientInvokers = InvokerRegistry.getClientInvokers();
      assertEquals(0, clientInvokers.length);
   }

   public void testRemoteClientInvokerDisconnect() throws Throwable
   {
      String passByValueServerURI =  serverURI + "?byvalue=true";
      Client client = new Client(new InvokerLocator(passByValueServerURI));

      ClientInvoker[] clientInvokers = InvokerRegistry.getClientInvokers();
      assertEquals(1, clientInvokers.length);

      RemoteClientInvoker clientInvoker = (RemoteClientInvoker)clientInvokers[0];
      assertEquals(passByValueServerURI, clientInvoker.getLocator().getLocatorURI());


      client.disconnect();


      clientInvokers = InvokerRegistry.getClientInvokers();
      assertEquals(0, clientInvokers.length);
   }


   // Package protected ---------------------------------------------

   // Protected -----------------------------------------------------

   // Private -------------------------------------------------------

   // Inner classes -------------------------------------------------

}
