/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/
package org.jboss.test.remoting.lifecycle;

import org.jboss.remoting.InvalidConfigurationException;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.transport.Connector;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:tom@jboss.org">Tom Elrod</a>
 */
public class InvokerLifecycleTestCase extends TestCase
{
   public InvokerLifecycleTestCase(String name)
   {
      super(name);
   }

   public void testMultipleConnectors() throws Exception
   {
      InvokerLocator serverLocator = new InvokerLocator("socket://localhost:2222");
      Connector connector1 = new Connector();
      connector1.setInvokerLocator(serverLocator.getLocatorURI());
      connector1.start();

      Connector connector2 = new Connector();
      connector2.setInvokerLocator(serverLocator.getLocatorURI());

      try
      {
         connector2.start();
      }
      catch(InvalidConfigurationException ice)
      {
         assertTrue("Got InvalidConfigurationException as expected.", true);
         return;
      }
      finally
      {
         connector1.stop();
         connector2.stop();
      }

      assertTrue("Did not get InvalidConfiguration which was NOT expected.", false);
   }

   public void testNonConcurrentConnectors() throws Exception
   {
      InvokerLocator serverLocator = new InvokerLocator("socket://localhost:2222");
      Connector connector1 = new Connector();
      connector1.setInvokerLocator(serverLocator.getLocatorURI());
      connector1.start();
      connector1.stop();

      Connector connector2 = new Connector();
      connector2.setInvokerLocator(serverLocator.getLocatorURI());

      try
      {
         connector2.start();
      }
      catch(InvalidConfigurationException ice)
      {
         assertTrue("Got InvalidConfigurationException which was unexpected.", false);
         return;
      }
      finally
      {
         connector2.stop();
      }

      assertTrue("Did not get InvalidConfiguration which is as expected.", true);

   }
}