/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.marshall.dynamic.remote.http;

import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:telrod@e2technologies.net">Tom Elrod</a>
 */
public class MarshallerLoadingClient extends TestCase implements MarshallerLoadingConstants
{
   public void testInvocation() throws Throwable
   {
      Client remotingClient = null;

      // first, make sure can not load marshaller from current classpath
      try
      {
         Class.forName("org.jboss.test.remoting.marshall.dynamic.remote.http.TestMarshaller");
         assertTrue("Was able to load the TestMarshaller class locally, when it should NOT be on the classpath.", false);
         return;
      }
      catch(ClassNotFoundException e)
      {
         assertTrue("Was not able to load TestMarshaller from local classpath as expected.", true);
      }

      try
      {
         InvokerLocator locator = new InvokerLocator(locatorURI);
         System.out.println("Calling remoting server with locator uri of: " + locatorURI);

         remotingClient = new Client(locator, null);
         Object response = remotingClient.invoke("Do something", null);
         assertTrue(true);

         System.out.println("Invocation response: " + response);
      }
      finally
      {
         if(remotingClient != null)
         {
            remotingClient.disconnect();
         }
      }
   }

   public static void main(String[] args)
   {
      MarshallerLoadingClient client = new MarshallerLoadingClient();
      try
      {
         client.testInvocation();
      }
      catch(Throwable throwable)
      {
         throwable.printStackTrace();
      }
   }

   /**
    * Can pass transport and port to be used as parameters.
    * Valid transports are 'rmi' and 'socket'.
    *
    * @param args
    */
//   public static void main(String[] args)
//   {
//      org.apache.log4j.BasicConfigurator.configure();
//      org.apache.log4j.Category.getRoot().setLevel(Level.INFO);
//      org.apache.log4j.Category.getInstance("org.jboss.remoting").setLevel(Level.DEBUG);
//
//      /*
//      if(args != null && args.length == 2)
//      {
//         transport = args[0];
//         port = Integer.parseInt(args[1]);
//      }
//      */
//      MarshallerLoadingClient client = new MarshallerLoadingClient(MarshallerLoadingClient.class.getName());
//      try
//      {
//         MultipleTestRunner runner = new MultipleTestRunner();
//         runner.doRun(client, true);
//      }
//      catch(Throwable e)
//      {
//         e.printStackTrace();
//         System.exit(1);
//      }
//      System.exit(0);
//   }
//

}