/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.oneway;

import java.util.ArrayList;
import java.util.List;
import javax.management.MBeanServer;
import org.jboss.logging.Logger;
import org.jboss.remoting.InvocationRequest;
import org.jboss.remoting.ServerInvocationHandler;
import org.jboss.remoting.ServerInvoker;
import org.jboss.remoting.callback.InvokerCallbackHandler;
import org.jboss.remoting.invocation.RemoteInvocation;

/**
 * MockServerInvocationHandler
 *
 * @author <a href="mailto:jhaynie@vocalocity.net">Jeff Haynie</a>
 * @author <a href="mailto:telrod@e2technologies.net">Tom Elrod</a>
 * @version $Revision: 1.3 $
 */
public class OnewayServerInvocationHandler implements ServerInvocationHandler
{
   private ServerInvoker invoker;
   private List listeners = new ArrayList();
   private static final Logger log = Logger.getLogger(OnewayServerInvocationHandler.class);

   private Object lastParam = null;

   /**
    * set the invoker that owns this handler
    *
    * @param invoker
    */
   public void setInvoker(ServerInvoker invoker)
   {
      this.invoker = invoker;
   }

   /**
    * set the mbean server that the handler can reference
    *
    * @param server
    */
   public void setMBeanServer(MBeanServer server)
   {
   }

   public Object invoke(InvocationRequest invocation)
         throws Throwable
   {
      Object param = invocation.getParameter();
      String methodName = "";
      Object[] params = null;
      String[] sig = null;

      if(param instanceof RemoteInvocation)
      {
         RemoteInvocation rminvo = (RemoteInvocation) param;
         methodName = rminvo.getMethodName();
         params = rminvo.getParameters();
      }
      else
      {
         throw new Exception("Unknown invocation payload (" + param + ").  " +
                             "Should be instance of RemoteInvocation.");
      }

      Object ret = null;
      if(methodName.equals("saveInvocationParameter"))
      {
         if(params != null)
         {
            lastParam = params[0];
            ret = lastParam;
         }
      }
      else if(methodName.equals("getLastInvocationParameter"))
      {
         ret = lastParam;
      }
      else
      {
         log.error("Expected parameter to be either 'saveInvocationParameter' or 'getLastInvocationParameter'.");
      }
      return ret;
   }


   public void addListener(InvokerCallbackHandler callbackHandler)
   {
      listeners.add(callbackHandler);
      log.debug("added listener " + callbackHandler);
   }

   public void removeListener(InvokerCallbackHandler callbackHandler)
   {
      listeners.remove(callbackHandler);
      log.debug("removed listener " + callbackHandler);
   }

}
