/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.performance.synchronous;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import org.jboss.jrunit.extensions.ServerTestCase;
import org.jboss.logging.Logger;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.ServerInvocationHandler;
import org.jboss.remoting.transport.Connector;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class PerformanceServerTest extends ServerTestCase
{
   // default transport and port
   private String transport = "socket";
   private int serverPort = 9090;

   private Connector connector = null;

   private static final Logger log = Logger.getLogger(PerformanceServerTest.class);

   public void init(Map metatdata) throws Exception
   {
      if(serverPort < 0)
      {
         //serverPort = Math.abs(new Random().nextInt(2000) + 2000);
         throw new Exception("port setting for server can not be null.");
      }
      log.debug("port = " + serverPort);

      connector = new Connector();
      InvokerLocator locator = new InvokerLocator(buildLocatorURI(metatdata));
      connector.setInvokerLocator(locator.getLocatorURI());
      connector.create();
      connector.addInvocationHandler(getSubsystem(), getServerInvocationHandler());
      connector.start();
      log.info("Server started on " + locator.getLocatorURI());
   }

   private String buildLocatorURI(Map metadata)
   {
      if(metadata == null || metadata.size() == 0)
      {
         return transport + "://localhost:" + serverPort;
      }
      else
      {
         StringBuffer uriBuffer = new StringBuffer(transport + "://localhost:" + serverPort);

         Set keys = metadata.keySet();
         Iterator itr = keys.iterator();
         while(itr.hasNext())
         {
            String key = (String) itr.next();
            String value = (String) metadata.get(key);
            uriBuffer.append(key + "=" + value + ",");
         }
         return uriBuffer.substring(0, uriBuffer.length() - 1);
      }
   }

   protected String getSubsystem()
   {
      return "performance";
   }

   protected ServerInvocationHandler getServerInvocationHandler()
   {
      return new PerformanceInvocationHandler();
   }

   protected void setUp() throws Exception
   {
      String newTransport = System.getProperty(PerformanceTestCase.REMOTING_TRANSPORT);
      if(newTransport != null && newTransport.length() > 0)
      {
         transport = newTransport;
         log.info("Using transport: " + transport);
      }

      init(null);
   }

   protected void tearDown() throws Exception
   {
      if(connector != null)
      {
         connector.stop();
         connector.destroy();
      }
   }

   public static void main(String[] args)
   {
      PerformanceServerTest test = new PerformanceServerTest();
      try
      {
         test.setUp();
         Thread.currentThread().sleep(300000);
         test.tearDown();
      }
      catch(Exception e)
      {
         e.printStackTrace();
      }
   }
}