/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.performance.synchronous;

import java.io.IOException;
import org.apache.log4j.Level;
import org.jboss.jrunit.harness.TestDriver;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class PerformanceTestCase extends TestDriver
{
   protected int numberOfClients = 1;

   public static final String REMOTING_TRANSPORT = "remoting.transport";
   public static final String PAYLOAD_SIZE = "remoting.payload.size";
   public static final String NUMBER_OF_CLIENTS = "remoting.number_of_clients";
   public static final String NUMBER_OF_CALLS = "remoting.number_of_calls";

   public void declareTestClasses()
   {
      //**************** LOGGING ***********************
      org.apache.log4j.BasicConfigurator.configure();
      org.apache.log4j.Category.getRoot().setLevel(Level.DEBUG);
      //org.apache.log4j.Category.getInstance("org.jboss.remoting").setLevel(Level.DEBUG);

      org.apache.log4j.SimpleLayout layout = new org.apache.log4j.SimpleLayout();

      try
      {
         org.apache.log4j.FileAppender fileAppender = new org.apache.log4j.FileAppender(layout, "debug_output.log");
         fileAppender.setThreshold(Level.DEBUG);
         fileAppender.setAppend(false);
         org.apache.log4j.Category.getRoot().addAppender(fileAppender);
      }
      catch(IOException e)
      {
         e.printStackTrace();
      }
      //*************** END LOGGING ***********************



      String numOfClients = System.getProperty(NUMBER_OF_CLIENTS);
      if(numOfClients != null && numOfClients.length() > 0)
      {
         try
         {
            numberOfClients = Integer.parseInt(numOfClients);
         }
         catch(NumberFormatException e)
         {
            e.printStackTrace();
         }
      }

      addTestClasses(PerformanceClientTest.class.getName(),
                     numberOfClients,
                     PerformanceServerTest.class.getName());
   }

   protected Level getTestHarnessLogLevel()
   {
      return Level.INFO;
      //return Level.DEBUG;
   }

   /**
    * The log level to run as for the test case.
    *
    * @return
    */
   protected Level getTestLogLevel()
   {
      return Level.INFO;
      //return Level.DEBUG;
   }

   /**
    * Returns the VM arguments to be passed to the vm when creating the client test cases (actually their harness).
    * The default value is null.
    *
    * @return
    */
   protected String getClientJVMArguments()
   {
      return getJVMArguments();
//      String args = "-Xdebug -Xnoagent -Djava.compiler=NONE -Xrunjdwp:transport=dt_socket,server=y,suspend=y,address=5000 ";
//      args = args + getJVMArguments();
//      return args;
   }

   /**
    * Returns the VM arguments to be passed to the vm when creating the server test cases (actually their harness).
    * The default value is null.
    *
    * @return
    */
   protected String getServerJVMArguments()
   {
      return getJVMArguments();
   }

   /**
    * Returns the VM arguments to be passed to the vm when creating the client and server test cases (actually their harness).
    * The default value is null.
    *
    * @return
    */
   private String getJVMArguments()
   {
      String vmArgs = "";

      String transport = System.getProperty(REMOTING_TRANSPORT);
      if(transport != null && transport.length() > 0)
      {
         vmArgs = "-D" + REMOTING_TRANSPORT + "=" + transport;
      }
      String payloadSize = System.getProperty(PAYLOAD_SIZE);
      if(payloadSize != null && payloadSize.length() > 0)
      {
         vmArgs = vmArgs + " -D" + PAYLOAD_SIZE + "=" + payloadSize;
      }
      String numOfCalls = System.getProperty(NUMBER_OF_CALLS);
      if(numOfCalls != null && numOfCalls.length() > 0)
      {
         vmArgs = vmArgs + " -D" + NUMBER_OF_CALLS + "=" + numOfCalls;
      }

      return vmArgs;
   }

   /**
    * How long to wait for test results to be returned from the client(s).  If goes longer than the
    * specified limit, will throw an exception and kill the running test cases.  Default value is
    * RESULTS_TIMEOUT.
    *
    * @return
    */
   protected long getResultsTimeout()
   {
      return 300000;
   }

   /**
    * How long for the server test case to wait for tear down message.  If exceeds timeout,
    * will throw exception.  The default value is TEARDOWN_TIMEOUT.
    *
    * @return
    */
   protected long getTearDownTimeout()
   {
      return 300000;
   }

   /**
    * How long to allow each of the test cases to run their tests.  If exceeds this timeout
    * will throw exception and kill tests.  The default value is RUN_TEST_TIMEOUT.
    *
    * @return
    */
   protected long getRunTestTimeout()
   {
      return 300000;
   }

}
