/**
 * JBoss, the OpenSource J2EE WebOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.registry;

import javax.management.MBeanServer;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvocationRequest;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.InvokerRegistry;
import org.jboss.remoting.ServerInvocationHandler;
import org.jboss.remoting.ServerInvoker;
import org.jboss.remoting.callback.InvokerCallbackHandler;
import org.jboss.remoting.transport.ClientInvoker;
import org.jboss.remoting.transport.Connector;
import org.jboss.remoting.transport.local.LocalClientInvoker;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:ovidiu@jboss.org">Ovidiu Feodorov</a>
 *
 * @version <tt>$Revision: 1.3 $</tt>
 *
 * $Id: InvokerRegistryTestCase.java,v 1.3 2005/06/20 03:38:45 telrod Exp $
 */
public class InvokerRegistryTestCase extends TestCase
{
   // Constants -----------------------------------------------------

   // Static --------------------------------------------------------

   // Attributes ----------------------------------------------------

   // Constructors --------------------------------------------------

   public InvokerRegistryTestCase(String name)
   {
      super(name);
   }

   // Public --------------------------------------------------------

   public void setUp() throws Exception
   {
      super.setUp();
   }

   public void tearDown() throws Exception
   {
      super.tearDown();
   }

   public void testEmptyInvokerRegistry() throws Throwable
   {
      ClientInvoker[] clientInvokers = InvokerRegistry.getClientInvokers();
      assertEquals(0, clientInvokers.length);
   }

   public void testInvokerRegistry() throws Throwable
   {
      String serverlocatorURI = "socket://127.0.0.1:5555";

      Connector server = new Connector();
      server.setInvokerLocator(serverlocatorURI);
      server.start();
      server.addInvocationHandler("TEST", new ServerInvocationHandlerImpl());

      new Client(new InvokerLocator(serverlocatorURI), "TEST");

      ClientInvoker[] clientInvokers = InvokerRegistry.getClientInvokers();
      assertEquals(1, clientInvokers.length);

      LocalClientInvoker clientInvoker = (LocalClientInvoker)clientInvokers[0];
      InvokerLocator locator = clientInvoker.getLocator();

      assertEquals("socket", locator.getProtocol());
      assertEquals("127.0.0.1", locator.getHost());
      assertEquals(5555, locator.getPort());

   }

   // Package protected ---------------------------------------------

   // Protected -----------------------------------------------------

   // Private -------------------------------------------------------

   // Inner classes -------------------------------------------------

   private class ServerInvocationHandlerImpl implements ServerInvocationHandler
   {
      public void setMBeanServer(MBeanServer server)
      {
      }

      public void setInvoker(ServerInvoker invoker)
      {
      }

      public Object invoke(InvocationRequest invocation) throws Throwable
      {
         return null;
      }

      public void addListener(InvokerCallbackHandler callbackHandler)
      {
      }

      public void removeListener(InvokerCallbackHandler callbackHandler)
      {
      }
   }
}
