/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.stream;

import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class StreamingTestClient extends TestCase
{
   // Default locator values
   private static String transport = "socket";
   private static String host = "localhost";
   private static int port = 5400;

   private String locatorURI = transport + "://" + host + ":" + port;
   private Client remotingClient = null;
   private File testFile = null;
   private FileInputStream fileInput = null;

   public void testStream() throws Throwable
   {
      URL fileURL = this.getClass().getResource("test.txt");
      if(fileURL == null)
      {
         throw new Exception("Can not find file test.txt");
      }
      testFile = new File(fileURL.getFile());
      fileInput = new FileInputStream(testFile);

      String param = "foobar";
      long fileLength = testFile.length();
      System.out.println("File size = " + fileLength);
      Object ret = remotingClient.invoke(fileInput, param);

      assertEquals(param, ret);

      Object response = remotingClient.invoke("get_size");
      int returnedFileLength = ((Integer)response).intValue();
      System.out.println("Invocation response: " + response);

      if(fileLength == returnedFileLength)
      {
         System.out.println("PASS");
      }
      else
      {
         System.out.println("FAILED - returned file length was " + returnedFileLength);
      }
      assertEquals(fileLength, returnedFileLength);
   }

   public void setUp() throws Exception
   {
      InvokerLocator locator = new InvokerLocator(locatorURI);
      System.out.println("Calling remoting server with locator uri of: " + locatorURI);

      remotingClient = new Client(locator);
   }

   public void tearDown() throws Exception
   {
      if(remotingClient != null)
      {
         remotingClient.disconnect();
      }
      if(fileInput != null)
      {
         fileInput.close();
      }
   }

   /**
    * Can pass transport and port to be used as parameters.
    * Valid transports are 'rmi' and 'socket'.
    *
    * @param args
    */
   public static void main(String[] args)
   {
      if(args != null && args.length == 2)
      {
         transport = args[0];
         port = Integer.parseInt(args[1]);
      }
      String locatorURI = transport + "://" + host + ":" + port;
      StreamingTestClient client = new StreamingTestClient();
      try
      {
         client.setUp();
         client.testStream();
         client.tearDown();
      }
      catch(Throwable e)
      {
         e.printStackTrace();
      }
   }


}