/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.transport.http.ssl.basic;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.transport.http.ssl.HTTPSClientInvoker;
import org.jboss.remoting.transport.http.HTTPServerInvoker;
import org.jboss.test.remoting.transport.http.HTTPInvokerTestServer;
import org.jboss.test.remoting.transport.http.ssl.SSLInvokerConstants;
import org.jboss.test.remoting.transport.web.ComplexObject;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class HTTPSInvokerTestClient extends TestCase implements SSLInvokerConstants
{
   public void testInvocationWithHeaders() throws Exception
   {
      Client remotingClient = null;

      try
      {
         // since doing basic (using default ssl server socket factory)
         // need to set the system properties to the truststore
         String trustStoreFilePath = this.getClass().getResource("../.truststore").getFile();
         System.setProperty("javax.net.ssl.trustStore", trustStoreFilePath);
         // set this property so does not do host verification
         System.setProperty(HTTPSClientInvoker.IGNORE_HTTPS_HOST, "true");

         String locatorURI = transport + "://" + host + ":" + port;
         InvokerLocator locator = new InvokerLocator(locatorURI);
         System.out.println("Calling remoting server with locator uri of: " + locatorURI);

         remotingClient = new Client(locator);

         Map metadata = new HashMap();
         metadata.put(Client.RAW, Boolean.TRUE);
         metadata.put("TYPE", "POST");

         Properties headerProps = new Properties();
         headerProps.put("SOAPAction", "http://www.example.com/fibonacci");
         headerProps.put("Content-type", "application/soap+xml");

         metadata.put("HEADER", headerProps);

         Object response = null;

         // test with null return expected
         response = remotingClient.invoke(HTTPInvokerTestServer.NULL_RETURN_PARAM, metadata);

         assertNull(response);

         response = remotingClient.invoke("Do something", metadata);

         assertEquals(HTTPInvokerTestServer.RESPONSE_VALUE, response);

         // test with small object
         headerProps.put("Content-type", HTTPServerInvoker.BINARY);
         response = remotingClient.invoke(new ComplexObject(2, "foo", true), metadata);

         assertEquals(HTTPSInvokerTestServer.OBJECT_RESPONSE_VALUE, response);

         // test with large object
         response = remotingClient.invoke(new ComplexObject(2, "foo", true, 8000), metadata);

         assertEquals(HTTPSInvokerTestServer.LARGE_OBJECT_RESPONSE_VALUE, response);

      }
      catch(Throwable throwable)
      {
         throw new Exception(throwable);
      }
      finally
      {
         if(remotingClient != null)
         {
            remotingClient.disconnect();
         }
      }


   }


}