
/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 *
 * Created on Aug 18, 2005
 */
package org.jboss.test.remoting.transport.multiplex;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Date;
import java.util.Random;

import org.jboss.logging.Logger;
import org.jboss.remoting.transport.multiplex.MultiplexingManager;
import org.jboss.remoting.transport.multiplex.VirtualSocket;

import junit.framework.TestCase;


/**
 * A BasicBehaviorClient.

 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 * @version $Revision: 1.6 $
 * <p>
 * Copyright (c) 2005
 * </p>
 */

public class BasicBehaviorClient extends TestCase implements MultiplexConstants
{
   protected static final Logger log = Logger.getLogger(BasicSocketBehaviorClient.class);
   protected static Socket scriptSocket;
   protected static Random random;
   protected static OutputStream scriptStream;
   protected String testName;
   protected Socket testSocket;
   protected InputStream is;
   protected OutputStream os;
   protected boolean basicBehaviorStarted = false;
   protected boolean testsOver = false;
   
   
   public void setUp() throws Exception
   {
      testName = getName();
      log.info("entering setUp() for test: " + testName);
      
      MultiplexingManager.setOnClient(true);
      
      try
      {
         if (scriptSocket == null)
         {
            scriptSocket = new VirtualSocket(basicBehaviorServerHost, basicBehaviorServerPort);
            scriptStream = scriptSocket.getOutputStream();
            log.info("BasicBehaviorClient: got message socket and message OutputStream");
         }
         
         testSocket = new VirtualSocket(basicBehaviorServerHost, basicBehaviorServerPort);
         log.info("BasicBehaviorClient: got test socket");
         
      }
      catch (Exception e)
      {
         log.error("unable to create socket", e);
         throw e;
      }  
      
      if (random == null)
      {
         random = new Random(new Date().getTime());
      }
      
      log.info("leaving setUp() for test: " + testName);
   }
   
   public void tearDown() throws Exception
   {
      log.info("entering tearDown() for test: " + testName);
      testSocket.close();
      
      if (testsOver)
         scriptSocket.close();
      
      log.info("leaving tearDown() for test: " + testName);
   }
}

