
/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 *
 * Created on Aug 18, 2005
 */

package org.jboss.test.remoting.transport.multiplex;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.BindException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.SocketException;
import java.net.UnknownHostException;

import javax.net.ServerSocketFactory;

import org.jboss.remoting.transport.multiplex.MultiPortVirtualServerSocket;
import org.jboss.remoting.transport.multiplex.MultiplexingManager;
import org.jboss.remoting.transport.multiplex.SinglePortVirtualServerSocket;
import org.jboss.remoting.transport.multiplex.VirtualServerSocketFactory;
import org.jboss.remoting.transport.multiplex.VirtualSocket;



/**
 * A BasicServerSocketBehaviorClient.

 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 * @version $Revision: 1.3 $
 * <p>
 * Copyright (c) 2005
 * </p>
 */

public class BasicServerSocketBehaviorClient extends BasicBehaviorClient implements MultiplexConstants
{
   
/***
 *  FIXME Comment this
 * 
 *
 */
   public void testPiggybackedSinglePortVirtualServerSocket()
   {
      boolean success = true;
      
      try
      {         
         byte[] script = new byte[] {READ, // make server wait until client's ServerSocket is up
                                     CONNECT_TO_CLIENT,
                                     WRITE_TO_CLIENT,
                                     READ_FROM_CLIENT,
                                     CLOSE_CLIENT_SOCKET,
                                     READ, // make server wait until client's ServerSocket is up
                                     CONNECT_TO_CLIENT,
                                     WRITE_TO_CLIENT,
                                     READ_FROM_CLIENT,
                                     CLOSE_CLIENT_SOCKET,
                                     READ, // make server wait until client's ServerSocket is up
                                     CONNECT_TO_CLIENT,
                                     WRITE_TO_CLIENT,
                                     READ_FROM_CLIENT,
                                     CLOSE_CLIENT_SOCKET, 
                                     CLOSE_TEST_SOCKET};
               
         scriptStream.write(script.length);
         scriptStream.write(script); 
      }
      catch (IOException e)
      {
         log.error(e);
      }
      
      SinglePortVirtualServerSocket serverSocket = null;
      int testSocketPort = testSocket.getLocalPort();
      
      try
      {
         serverSocket = new SinglePortVirtualServerSocket(testSocketPort);
         success &= doOneServerSocketTest(serverSocket, testSocketPort);
         serverSocket.close();
         success &= serverSocket.isBound() == true;
         success &= serverSocket.isClosed() == true;
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      try
      {
         serverSocket = new SinglePortVirtualServerSocket();
         success &= serverSocket.isBound() == false;
         success &= serverSocket.isClosed() == false;
         
         // Bind serverSocket to the testSocket's local port.  This will get the MultiplexingManager
         // associated with testSocket, which is already connected
         doBind(serverSocket, null, new InetSocketAddress(clientServerSocketHost, testSocketPort));
         
         success &= serverSocket.isBound() == true;
         success &= doOneServerSocketTest(serverSocket, testSocketPort);
         serverSocket.close();
         success &= serverSocket.isBound() == true;
         success &= serverSocket.isClosed() == true;
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      try
      {
         // Create a new SinglePortVirtualServerSocket, and bind it to testSocket's local port.
         // This will get the MultiplexingManager associated with testSocket, which is already connected
         serverSocket = doBind(null,
                               basicBehaviorServerSocketAddress,
                               new InetSocketAddress(clientServerSocketHost, testSocketPort));
       
         success &= serverSocket.isBound() == true;
         success &= serverSocket.isConnected() == true;
         success &= serverSocket.isClosed() == false;
         success &= doOneServerSocketTest(serverSocket, testSocketPort);
         serverSocket.close();
         success &= serverSocket.isBound() == true;
         success &= serverSocket.isConnected() == true;
         success &= serverSocket.isClosed() == true;
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      assertTrue(success);
   }
   
   
   public void testIndependentSinglePortVirtualServerSocket()
   {
      boolean success = true;
      
      try
      {         
         byte[] script = new byte[] {READ, // make server wait until client's ServerSocket is up
                                     CONNECT_TO_CLIENT_SINGLE_PORT,
                                     WRITE_TO_CLIENT,
		               				 READ_FROM_CLIENT,
		               				 CLOSE_CLIENT_SOCKET, 
		               				 READ, // make server wait until client's ServerSocket is up
		               				 CONNECT_TO_CLIENT_SINGLE_PORT,
                                     WRITE_TO_CLIENT,
		               				 READ_FROM_CLIENT,
		               				 CLOSE_CLIENT_SOCKET, 
		               			     CLOSE_TEST_SOCKET
		               			     };
         
         scriptStream.write(script.length);
         scriptStream.write(script);
      }
      catch (IOException e)
      {
         log.error(e);
      }
      
      SinglePortVirtualServerSocket serverSocket = null;
       
      try
      {
         serverSocket = new SinglePortVirtualServerSocket();
         success &= serverSocket.isBound() == false;
         success &= serverSocket.isClosed() == false;
         
         // Bind serverSocket to an unused local address.  This will lead to the creation of a new
         // real socket, bound to that address, and a MultiplexingManager to wrap it.
         doBind(serverSocket, null, new InetSocketAddress(clientServerSocketHost, clientServerSocketPort));
         
         success &= serverSocket.isBound() == true;
         
         // Connect serverSocket to a remote address.  This will cause the real socket to connect to a
         // MultiPortVirtualServerSocket at that address, leading to the creation of a real socket on the
         // remote host and a MultiplexingManager to wrap ti.
         serverSocket.connect(basicBehaviorServerSocketAddress);
         
         success &= serverSocket.isConnected();
         success &= doOneServerSocketTest(serverSocket, clientServerSocketPort);
         serverSocket.close();
         success &= serverSocket.isBound() == true;
         success &= serverSocket.isClosed() == true;
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      try
      {
         // Create a new SinglePortVirtualServerSocket, bind it to an unused local address, and connect it
         // to the server.  This will lead to the creation of a new real socket, wrapped by a MultiplexingManager,
         // bound to that address and connected to the server.
         serverSocket = doBind(null, basicBehaviorServerSocketAddress, clientServerSocketAddress);
         
         success &= serverSocket.isBound() == true;
         success &= serverSocket.isConnected() == true;
         success &= serverSocket.isClosed() == false;
         success &= doOneServerSocketTest(serverSocket, clientServerSocketPort);
         serverSocket.close();
         success &= serverSocket.isBound() == true;
         success &= serverSocket.isConnected() == true;
         success &= serverSocket.isClosed() == true;
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      assertTrue(success);
   }
   
   
/**
 *  FIXME Comment this
 * 
 *
 */
   public void testMultiPortVirtualServerSocket()
   {
      boolean success = true;
      
      try
      {         
         byte[] script = new byte[] {READ, // make server wait until client's ServerSocket is up
                                     CONNECT_TO_CLIENT_MULTI_PORT,
                                     WRITE_TO_CLIENT,
                                     READ_FROM_CLIENT,
                                     CLOSE_CLIENT_SOCKET,  
                                     CLOSE_TEST_SOCKET
         };
         
         scriptStream.write(script.length);
         scriptStream.write(script);
      }
      catch (IOException e)
      {
         log.error(e);
      }
      
      MultiPortVirtualServerSocket serverSocket = null;
      
      try
      {
         InetAddress bindAddress = InetAddress.getByName(clientServerSocketHost);
         MultiplexingManager.setOnClient(false);
         serverSocket = new MultiPortVirtualServerSocket(clientServerSocketPort, 50, bindAddress);
         MultiplexingManager.setOnClient(true);
         success &= doOneServerSocketTest(serverSocket, clientServerSocketPort);
         serverSocket.close();
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      assertTrue(success);
   }
   
   
/**
 *  FIXME Comment this
 * 
 *
 */
   public void testServerSocketFactory()
   { 
      log.info("entering testServerSocketFactory()");
      boolean success = true;

      try
      {         
         byte[] script = new byte[]
         {
               READ, // make server wait until client's ServerSocket is up
               CONNECT_TO_CLIENT_SINGLE_PORT,
               WRITE_TO_CLIENT,
               READ_FROM_CLIENT,
               CLOSE_CLIENT_SOCKET, 
               READ, // make server wait until client's ServerSocket is up
               CONNECT_TO_CLIENT_MULTI_PORT,
               WRITE_TO_CLIENT,
               READ_FROM_CLIENT,
               CLOSE_CLIENT_SOCKET, 
               CLOSE_TEST_SOCKET
         };
         
         scriptStream.write(script.length);
         scriptStream.write(script); 
      }
      catch (IOException e)
      {
         log.error(e);
      }
      
      try
      {
         ServerSocketFactory serverSocketFactory = VirtualServerSocketFactory.getDefault();
         MultiplexingManager.setOnClient(true);
         SinglePortVirtualServerSocket serverSocket
         	= (SinglePortVirtualServerSocket) serverSocketFactory.createServerSocket(clientServerSocketPort);
         doBind(serverSocket, null, clientServerSocketAddress);
         serverSocket.connect(basicBehaviorServerSocketAddress);
         success &= doOneServerSocketTest(serverSocket, clientServerSocketPort);
         serverSocket.close();
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      try
      {
         ServerSocketFactory serverSocketFactory = VirtualServerSocketFactory.getDefault();
         MultiplexingManager.setOnClient(false);
         InetAddress bindAddress = InetAddress.getByName(clientServerSocketHost);
         MultiPortVirtualServerSocket serverSocket
         	= (MultiPortVirtualServerSocket) serverSocketFactory.createServerSocket(clientServerSocketPort, 50, bindAddress);
         success &= doOneServerSocketTest(serverSocket, clientServerSocketPort);
         serverSocket.close();
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      assertTrue(success);
   }
   
   
/**
 *  FIXME Comment this
 * 
 * @param serverSocket
 * @param localPort
 */
   protected boolean doOneServerSocketTest(final ServerSocket serverSocket, int localPort)
   {
      log.debug("entering doOneServerSocketTest");
      boolean success = true;
      InetAddress clientServerSocket_Address = null;
      InetSocketAddress clientServerSocket_SocketAddress = null;
      
      try
      {
         clientServerSocket_Address = InetAddress.getByName(clientServerSocketHost);
         clientServerSocket_SocketAddress = new InetSocketAddress(clientServerSocket_Address, localPort);
         
      }
      catch (UnknownHostException e)
      {
         log.error(e);
         fail();
      }
      
      success &= serverSocket.isBound() == true;
      success &= serverSocket.isClosed() == false;
      success &= serverSocket.getInetAddress().equals(clientServerSocket_Address);
      success &= serverSocket.getLocalSocketAddress().equals(clientServerSocket_SocketAddress);
      success &= serverSocket.getLocalPort() == localPort;
   
      try
      {
         class StartSocketThread extends Thread
         {      
            public VirtualSocket clientSocket;
            
            public void run()
            {
               try
               {
                  clientSocket = (VirtualSocket) serverSocket.accept();
               }
               catch (Exception e)
               {
                  log.error(e);
               }
            }
         }
         
         StartSocketThread startSocketThread = new StartSocketThread();
         startSocketThread.start();
         os = testSocket.getOutputStream();
         os.write(3); // server is waiting to read, after which it will connect

         VirtualSocket clientSocket = startSocketThread.clientSocket;
         
         while (clientSocket == null)
         {
            try
            {
               Thread.sleep(500);
            }
            catch (Exception e)
            {
               log.error(e);
            }
         
            clientSocket = startSocketThread.clientSocket;
         }

         log.info("VirtualServerSocket executed accept()");
         InputStream is_client = clientSocket.getInputStream();
         OutputStream os_client = clientSocket.getOutputStream();
         int b = is_client.read();
         os_client.write(b);
         clientSocket.close();
         serverSocket.close();
      }
      catch (IOException e)
      {
         log.error(e);
         fail();
      }
      
      log.debug("leaving doOneServerSocketTest");
      return success;
   }
   
   
   protected SinglePortVirtualServerSocket doBind(SinglePortVirtualServerSocket serverSocket,
                                                  InetSocketAddress remoteAddress,
                                                  InetSocketAddress localAddress)
   {
      log.info("entering doBind()");
      log.info("serverSocket = " + serverSocket);
      SinglePortVirtualServerSocket newServerSocket = null;
      
      for (int i = 0; i < 5; i++)
      {
         try
         {
            if (serverSocket == null)
            {
               if (remoteAddress == null)
                  newServerSocket = new SinglePortVirtualServerSocket(localAddress.getPort());
               else
                  newServerSocket = new SinglePortVirtualServerSocket(remoteAddress, localAddress);  
            }
            else
            {
               // ServerSockets created in testServerSocketFactory() may come in already bound.
               if (!serverSocket.isBound())
                  serverSocket.bind(localAddress);
            }
            
            break;
         }
         catch (BindException e)
         {
            log.info("bind attempt failed: try again");
            
            if (newServerSocket != null)
            {
               try
               {
                  newServerSocket.close();
                  newServerSocket = null;
               }
               catch (IOException ignored)
               {
                  log.error(e);
               }
            }
            
            try
            {
               Thread.sleep(2000);
            }
            catch (InterruptedException ignored) {}
         }
         catch (SocketException e)
         {
            String message = e.getMessage();
            
            if (message.equals("Already bound") || message.equals("Cannot assign requested address"))
            {
               log.info("bind attempt failed: try again");
               log.info(e);
               
               if (newServerSocket != null)
               {
                  try
                  {
                     log.info("calling newServerSocket.close()");
                     newServerSocket.close();
                     newServerSocket = null;
                  }
                  catch (IOException ignored)
                  {
                     log.error(e);
                  }
               }

               try
               {
                  Thread.sleep(2000);
               }
               catch (InterruptedException ignored) {}
            }
            else
            {
               log.error(e);
               fail();
            }
         }
         catch (Exception e)
         {
            log.error(e);
            e.printStackTrace();
            fail();
         }
      }
      
      if (newServerSocket != null)
         serverSocket = newServerSocket;
            
      if (serverSocket == null || !serverSocket.isBound())
         fail();
      
      return serverSocket;
   }
}

