
/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 *
 * Created on Aug 13, 2005
 */
package org.jboss.test.remoting.transport.multiplex;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.BindException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;

import javax.net.SocketFactory;

import org.jboss.remoting.transport.multiplex.VirtualSocket;
import org.jboss.remoting.transport.multiplex.VirtualSocketFactory;


/**
 * A BasicBehaviorClient.
 
 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 * @version $Revision: 1.3 $
 * <p>
 * Copyright (c) 2005
 * </p>
 */

public class BasicSocketBehaviorClient extends BasicBehaviorClient implements MultiplexConstants
{  
/**
 *  FIXME Comment this
 * 
 *
 */
   public void testVirtualSocketConstructors()
   {
      log.info("entering testVirtualSocketConstructors()");
      boolean success = true;
      
      try
      {
         // This script is for server thread created for testSocket, 
         // which we don't use in this test.
         byte[] script = new byte[] {CLOSE_TEST_SOCKET};
         scriptStream.write(script.length);
         scriptStream.write(script);
      }
      catch (IOException e1)
      {
         log.error(e1);
      }
      
      try
      {         
         while (true)
         {
            try
            {
               int localPort = random.nextInt() & 0xffff;
               VirtualSocket socket1 = new VirtualSocket();
               socket1.bind(new InetSocketAddress(localPort));
               socket1.connect(basicBehaviorServerSocketAddress);
               success &= doOneConstructorTest(socket1, localPort);
               break;
            }
            catch (BindException e)
            {
               log.info("looking for a free socket");
            }
         }
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         VirtualSocket socket2 = new VirtualSocket(basicBehaviorServerHost , basicBehaviorServerPort);
         success &= doOneConstructorTest(socket2, socket2.getLocalPort());
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
           
      try
      {
         VirtualSocket socket3 = new VirtualSocket(basicBehaviorServerHost , basicBehaviorServerPort, false);
         fail();
      }
      catch (SocketException e)
      {
         success &= e.getMessage().equals("Deprecated: use DataGramSocket instead of stream = false");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
           
      try
      {
         VirtualSocket socket4 = new VirtualSocket(basicBehaviorServerHost , basicBehaviorServerPort, true);
         success &= doOneConstructorTest(socket4, socket4.getLocalPort());
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
     
      try
      {
         InetAddress inetAddress = InetAddress.getByName(basicBehaviorServerHost);
         VirtualSocket socket5 = new VirtualSocket(inetAddress , basicBehaviorServerPort);
         success &= doOneConstructorTest(socket5, socket5.getLocalPort());
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         InetAddress inetAddress = InetAddress.getByName(basicBehaviorServerHost);
         VirtualSocket socket6 = new VirtualSocket(inetAddress , basicBehaviorServerPort, false);
         fail();
      }
      catch (SocketException e)
      {
         success &= e.getMessage().equals("Deprecated: use DataGramSocket instead of stream = false");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         InetAddress inetAddress = InetAddress.getByName(basicBehaviorServerHost);
         VirtualSocket socket7 = new VirtualSocket(inetAddress , basicBehaviorServerPort, true);
         success &= doOneConstructorTest(socket7, socket7.getLocalPort());
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         while (true)
         {
            try
            {
               int localPort = random.nextInt() & 0xffff;
               InetAddress inetAddress = InetAddress.getByName("localhost");
               VirtualSocket socket8 = new VirtualSocket(basicBehaviorServerHost, basicBehaviorServerPort, inetAddress , localPort);
               success &= doOneConstructorTest(socket8, localPort);
               break;
            }
            catch (BindException e)
            {
               log.info("looking for a free socket");
            }
         }
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {         
         while (true)
         {
            try
            {
               int localPort = random.nextInt() & 0xffff;
               InetAddress remoteAddress = InetAddress.getByName(basicBehaviorServerHost);
               InetAddress localAddress = InetAddress.getByName("localhost");
               VirtualSocket socket9 = new VirtualSocket(remoteAddress, basicBehaviorServerPort, localAddress , localPort);
               success &= doOneConstructorTest(socket9, localPort);
               break;
            }
            catch (BindException e)
            {
               log.info("looking for a free socket");
            }
         }
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
   
      assertTrue(success);
   }
   
   
/**
 *  FIXME Comment this
 * 
 *
 */
   protected boolean doOneConstructorTest(VirtualSocket socket, int localPort)
   {
      boolean success = true;
      
      try
      {
         byte[] script = new byte[] {READ, CLOSE_TEST_SOCKET};
         scriptStream.write(script.length);
         scriptStream.write(script);
         
      }
      catch (IOException e1)
      {
         log.error(e1);
      }
      
      success &= socket.isBound();
      success &= socket.isConnected();
      success &= socket.getLocalPort() == localPort;
      
      success &= socket.getPort() == basicBehaviorServerPort;
      
      try
      {
         InetAddress localAddress = InetAddress.getByName("localhost");
         InetSocketAddress localSocketAddress = new InetSocketAddress(localAddress, localPort);
         success &= socket.getLocalAddress().equals(localAddress);
         success &= socket.getLocalSocketAddress().equals(localSocketAddress);
      }
      catch (UnknownHostException e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         InetAddress remoteAddress = InetAddress.getByName(basicBehaviorServerHost);
         success &= socket.getInetAddress().equals(remoteAddress);
         success &= socket.getRemoteSocketAddress().equals(basicBehaviorServerSocketAddress);
      }
      catch (UnknownHostException e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         // end script on server
         os = socket.getOutputStream();
         os.write(3);
      }
      catch (IOException e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         socket.close();
      }
      catch (IOException e)
      {
         log.error(e);
         fail();
      }

      return success;
   }
   
   
   /**
    *  FIXME Comment this
    * 
    * @throws Exception
    */
   public void testLocalInputShutdown() throws Exception
   {
      log.info("entering testLocalInputShutdown()");
      boolean success = true;
      
      try
      {
         byte[] script = new byte[] {READ, WRITE, READ, READ, CLOSE_TEST_SOCKET};
         scriptStream.write(script.length);
         scriptStream.write(script);  
         is = testSocket.getInputStream();
         os = testSocket.getOutputStream();
         os.write(3);
         int b = is.read();
         success &= b == 3;
         testSocket.shutdownInput();
         success &= testSocket.isConnected();
         success &= testSocket.isBound();
         success &= !testSocket.isClosed();
         success &= testSocket.isInputShutdown();
         success &= !testSocket.isOutputShutdown();
         b = is.read();
         success &= b == -1;
         os.write(5);
         os = testSocket.getOutputStream();
         os.write(7);
         testSocket.shutdownOutput();
      }
      catch (Throwable t)
      {
         log.error(t);
         fail();
      }
      
      try
      {
         testSocket.getInputStream();
         fail();
      }
      catch (SocketException e)
      {
         success &= e.getMessage().equals("Socket input is shutdown");
      }
      catch (Throwable t)
      {
         log.error(t);
         fail();
      }
      
      try
      {
         testSocket.shutdownInput();
         fail();
      }
      catch (SocketException e)
      {
         success &= e.getMessage().equals("Socket input is already shutdown");
      }
      catch (Throwable t)
      {
         log.error(t);
         fail();
      }
      
      assertTrue(success); 
   }
   
   public void testLocalOutputShutdown() throws Exception
   {
      log.info("entering testLocalOutputShutdown()");
      boolean success = true;
      
      try
      {
         byte[] script = new byte[] {READ, WRITE, READ, WRITE, CLOSE_TEST_SOCKET};
         scriptStream.write(script.length);
         scriptStream.write(script);  
         is = testSocket.getInputStream();
         os = testSocket.getOutputStream();
         os.write(3);
         os.write(5);
         int b = is.read();
         success &= b == 3;
         testSocket.shutdownOutput();
         success &= testSocket.isConnected();
         success &= testSocket.isBound();
         success &= !testSocket.isClosed();
         success &= !testSocket.isInputShutdown();
         success &=  testSocket.isOutputShutdown();
         is = testSocket.getInputStream();
         b = is.read();
         success &= b == 5;
         testSocket.shutdownInput();
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         // Note.  Real sockets allow the first write to succeed.  Is this important?
         os.write(5);
         log.error("write() should have failed");
         fail();
      }
      catch (SocketException e)
      {
         log.info(e);
         success &= e.getMessage().equals("Broken pipe");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         testSocket.getInputStream();
         log.error("getInputStream() should have failed");
         fail();
      }
      catch (SocketException e)
      {
         log.info(e);
         success &= e.getMessage().equals("Socket input is shutdown");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         testSocket.shutdownOutput();
         log.error("shutdownOutput() should have failed");
         fail();
      }
      catch (SocketException e)
      {
         log.info(e);
         success &= e.getMessage().equals("Socket output is already shutdown");
      }
      
      assertTrue(success); 
   }
   
   
   /**
    *  FIXME Comment this
    * 
    * @throws Exception
    */
   public void testLocalSocketClose() throws Exception
   {
      log.info("entering testLocalSocketClose()");
      boolean success = true;
      
      try
      {
         byte[] script = new byte[] {READ, WRITE, CLOSE_TEST_SOCKET};
         scriptStream.write(script.length);
         scriptStream.write(script); 
         is = testSocket.getInputStream();
         os = testSocket.getOutputStream();
         os.write(3);
         success &= (is.read() == 3);
         testSocket.close();
         success &= testSocket.isConnected();
         success &= testSocket.isBound();
         success &= testSocket.isClosed();
         success &= !testSocket.isInputShutdown();
         success &= !testSocket.isOutputShutdown();
         testSocket.close();
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         int b = is.read();
         // read() should have failed
         fail();
      }
      catch (SocketException e)
      {
         log.info(e);
         success &= e.getMessage().equals("Socket closed");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         // Note.  Real sockets allow the first write to succeed.  Is this important?
         os.write(7);
         log.error("write() should have failed");
         fail();
      }
      catch (SocketException e)
      {
         log.info(e);
         success &= e.getMessage().equals("Socket closed");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         testSocket.getInputStream();
         log.error("getInputStream() should have failed");
         fail();
      }
      catch (SocketException e)
      {
         log.info(e);
         success &= e.getMessage().equals("Socket is closed");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         testSocket.getOutputStream();
         log.error("getOutputStream() should have failed");
         fail();
      }
      catch (SocketException e)
      {
         log.info(e);
         success &= e.getMessage().equals("Socket is closed");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         testSocket.shutdownInput();
         log.error("shutdownInput() should have failed");
         fail();
      }
      catch (SocketException e)
      {
         log.info(e);
         success &= e.getMessage().equals("Socket is closed");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      try
      {
         testSocket.shutdownOutput();
         log.error("shutdownOutput() should have failed");
         fail();
      }
      catch (SocketException e)
      {
         log.info(e);
         success &= e.getMessage().equals("Socket is closed");
      }
      catch (Exception e)
      {
         log.error(e);
         fail();
      }
      
      assertTrue(success); 
   }
   
   /**
    *  FIXME Comment this
    * 
    * @throws Exception
    */
   public void testRemoteInputShutdown() throws Exception
   {
      log.info("entering testRemoteInputShutdown()");
      boolean success = true;
      
      try
      {
         byte[] script = new byte[] {READ, WRITE, READ, SHUTDOWN_INPUT, WRITE, CLOSE_TEST_SOCKET};
         scriptStream.write(script.length);
         scriptStream.write(script); 
         is = testSocket.getInputStream();
         os = testSocket.getOutputStream();
         os.write(3);
         int b = is.read();
         success &= b == 3;
         success &= testSocket.isConnected();
         success &= testSocket.isBound();
         success &= !testSocket.isClosed();
         success &= !testSocket.isInputShutdown();
         success &= !testSocket.isOutputShutdown();
         os.write(5);
         // remote socket will shut down input after reading 5
         success &= testSocket.isConnected();
         success &= testSocket.isBound();
         success &= !testSocket.isClosed();
         success &= !testSocket.isInputShutdown();
         success &= !testSocket.isOutputShutdown();  
         b = is.read();
         success &= b == 5;
         is = testSocket.getInputStream();
         os = testSocket.getOutputStream();
      }
      catch (Throwable t)
      {
         log.error(t);
         fail();
      }
      
      try
      {
         // it might take a while for remote input shutdown to take effect
         for (int i = 0; i < 5; i++)
         {
            os.write(7);
            Thread.sleep(1000);
         }
         
         // write() should have failed
         log.error("remote input shutdown: still able to write() after tries");
         fail();
      }
      catch (SocketException e)
      {
         success &= e.getMessage().equals("Broken pipe");
      }
      catch (Throwable t)
      {
         log.error(t);
         fail();
      }
      
      try
      {
         testSocket.shutdownInput();
         testSocket.shutdownOutput();
         testSocket.close();
      }
      catch (Throwable t)
      {
         log.error(t);
         fail();
      }
      
      assertTrue(success); 
   }
   
   
   /**
    *  FIXME Comment this
    * 
    * @throws Exception
    */
   public void testRemoteOutputShutdown() throws Exception
   {
      log.info("entering testRemoteOutputShutdown()");
      boolean success = true;
      
      try
      {
         byte[] script = new byte[] {READ, WRITE, READ, WRITE, READ, SHUTDOWN_OUTPUT, READ, CLOSE_TEST_SOCKET};
         scriptStream.write(script.length);
         scriptStream.write(script); 
         is = testSocket.getInputStream();
         os = testSocket.getOutputStream();
         os.write(3);
         int b = is.read();
         success &= b == 3;
         os.write(5);
         success &= testSocket.isConnected();
         success &= testSocket.isBound();
         success &= !testSocket.isClosed();
         success &= !testSocket.isInputShutdown();
         success &= !testSocket.isOutputShutdown();
         os.write(7);
         // remote socket will shut down output after reading 7
         success &= testSocket.isConnected();
         success &= testSocket.isBound();
         success &= !testSocket.isClosed();
         success &= !testSocket.isInputShutdown();
         success &= !testSocket.isOutputShutdown();  
         b = is.read();
         success &= b == 5;
         is = testSocket.getInputStream();
         os = testSocket.getOutputStream();
         b = is.read();
         success &= b == -1;
         os.write(9);
      }
      catch (Throwable t)
      {
         log.error(t);
         fail();
      }
      
      assertTrue(success); 
   }
   
   
/**
 *  FIXME Comment this
 * 
 * @throws Exception
 */
   public void testRemoteSocketClose() throws Exception
   {
      log.info("entering testRemoteSocketClose()");
      boolean success = true;
      
      try
      {
         byte[] script = new byte[] {READ, WRITE, CLOSE_TEST_SOCKET};
         scriptStream.write(script.length);
         scriptStream.write(script); 
         is = testSocket.getInputStream();
         os = testSocket.getOutputStream();
         success &= testSocket.isConnected();
         success &= testSocket.isBound();
         success &= !testSocket.isClosed();
         success &= !testSocket.isInputShutdown();
         success &= !testSocket.isOutputShutdown();
         os.write(3);
         // give remote socket time to close
         Thread.sleep(4000);
         int b = is.read();
         success &= b == 3;
         success &= testSocket.isConnected();
         success &= testSocket.isBound();
         success &= !testSocket.isClosed();
         success &= !testSocket.isInputShutdown();
         success &= !testSocket.isOutputShutdown();
         is = testSocket.getInputStream();
         os = testSocket.getOutputStream();
         b = is.read();
         success &= b == -1;
      }
      catch (Throwable t)
      {
         log.error(t);
         fail();
      }
      
      try
      {
         // remote socket should be closed by now, but just in case ...
         for (int i = 0; i < 5; i++)
         {
            os.write(5);
            Thread.sleep(1000);
         }
         
         // write() should have failed
         log.error("remote input shutdown: still able to write() after tries");
         fail();
      }
      catch (SocketException e)
      {
         success &= e.getMessage().equals("Broken pipe");
      }
      catch (Throwable t)
      {
         log.error(t);
         fail();
      }
      
      assertTrue(success); 
   }
   
   
/**
 *  FIXME Comment this
 * 
 *
 */
   public void testSocketFactory()
   {
      log.info("entering testSocketFactory()");
      boolean success = true;
      
      try
      {
         // This script is for server thread created for testSocket, 
         // which we don't use in this test.
         byte[] script = new byte[] {CLOSE_TEST_SOCKET};
         scriptStream.write(script.length);
         scriptStream.write(script);
      }
      catch (IOException e1)
      {
         log.error(e1);
      }
      
      SocketFactory socketFactory = VirtualSocketFactory.getDefault();
      Socket socket = null;
      
      try
      {
         socket = socketFactory.createSocket();
         socket.connect(basicBehaviorServerSocketAddress);
         success &= doOneSocketFactoryTest(socket);
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      
      try
      {
         socket = socketFactory.createSocket(basicBehaviorServerHost, basicBehaviorServerPort);
         success &= doOneSocketFactoryTest(socket);
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      try
      {
         int localPort = random.nextInt() & 0xffff;
         InetAddress inetAddress = InetAddress.getByName("localhost");
         socket = socketFactory.createSocket(basicBehaviorServerHost, basicBehaviorServerPort, inetAddress , localPort);
         success &= socket.getLocalAddress().equals(inetAddress);
         success &= socket.getLocalPort() == localPort;
         success &= doOneSocketFactoryTest(socket);
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      try
      {
         InetAddress inetAddress = InetAddress.getByName(basicBehaviorServerHost);
         socket = socketFactory.createSocket(inetAddress, basicBehaviorServerPort);
         success &= doOneSocketFactoryTest(socket);
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      
      try
      {  
         while (true)
         {
            try
            {
               InetAddress remoteAddress = InetAddress.getByName(basicBehaviorServerHost);
               InetAddress localAddress = InetAddress.getByName("localhost");
               int localPort = random.nextInt() & 0xffff;
               socket = socketFactory.createSocket(remoteAddress, basicBehaviorServerPort, localAddress , localPort);
               success &= socket.getLocalAddress().equals(localAddress);
               success &= socket.getLocalPort() == localPort;
               success &= doOneSocketFactoryTest(socket);
               break;
            }
            catch (BindException e)
            {
               log.info("looking for a free socket");
            }
         }
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      assertTrue(success);
   }
   
   
/**
 *  FIXME Comment this
 * 
 * @param socket
 * @return
 */
   protected boolean doOneSocketFactoryTest(Socket socket)
   {
      log.info("entering doOneFactoryTest()");
      boolean success = true;
      
      try
      {         
         byte[] script = new byte[]
                                  { 
               READ,
               WRITE,
               READ,
               CLOSE_TEST_SOCKET
                                  };
         
         scriptStream.write(script.length);
         scriptStream.write(script); 
      }
      catch (IOException e)
      {
         log.error(e);
      }
      
      InputStream is = null;
      OutputStream os = null;
      
      try
      {
         is = socket.getInputStream();
         os = socket.getOutputStream();
         os.write(3);
         success &= is.read() == 3;
         os.write(5);
         socket.close();
      }
      catch (Exception e)
      {
         log.error(e);
         e.printStackTrace();
         fail();
      }
      
      return success;
   }
}

