/*
 * Created on Jul 24, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.jboss.test.remoting.transport.multiplex;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;
import org.jboss.logging.Logger;
import org.jboss.remoting.transport.multiplex.SinglePortVirtualServerSocket;
import org.jboss.remoting.transport.multiplex.VirtualSocket;

/**
 *
 * <p>
 * Copyright (c) 2005
 * <p>
 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 */
public class ClientTest
{
   private static final Logger log = Logger.getLogger(ClientTest.class);

   private Socket socket = null;
   private ServerSocket serverSocket = null;

   private ObjectOutputStream oos = null;
   private ObjectInputStream  ois = null;

   /**
    *
    */
   public ClientTest()
   {
   }


   public static void main(String[] args)
   {
      new ClientTest().runTest();
   }


   protected void runTest()
   {
      log.info("Starting client test");

      try
      {
         socket = new VirtualSocket("127.0.0.1", 1999);
      }
      catch (UnknownHostException e)
      {
         log.error("UnknownHostException for \"localhost\"");
         e.printStackTrace();
      }
      catch (IOException e)
      {
         log.error("i/o exception creating VirtualSocket");
         e.printStackTrace();
      }

       new Thread()
       	   {
	          public void run()
	          {
	             testCallBacks();
	          }
       	   }.start();

      try
      {
         oos = new ObjectOutputStream(socket.getOutputStream());
         ois = new ObjectInputStream(socket.getInputStream());

         oos.write(3);
         oos.writeInt(7);
         oos.writeObject(new Integer(13));
         oos.writeObject("abc");
         oos.writeObject(new Boolean(false));
         oos.flush();

         int b = ois.read();
         int i0 =  ois.readInt();
         Integer i1 = (Integer) ois.readObject();
         String s = (String) ois.readObject();
         Boolean bool = (Boolean) ois.readObject();

         log.info("CLIENT: synchronous read byte     " + b);
         log.info("CLIENT: synchronous read int:     " + i0);
         log.info("CLIENT: synchronous read Integer: " + i1);
         log.info("CLIENT: synchronous read String:  " + s);
         log.info("CLIENT: synchronous read boolean: " + bool);
      }
      catch (IOException e)
      {
         log.error("i/o error", e);
         e.printStackTrace();
      }
      catch (ClassNotFoundException e)
      {
         log.error("class not found", e);
         e.printStackTrace();
      }

      try
      {
         socket.close();
      }
      catch (IOException ignore)
      {
         log.error("i/o exception closing socket");
      }

      log.info("Ending client test (main thread)");
   }


   protected void testCallBacks()
   {
      log.info("Starting client callback test thread");

      try
      {
         serverSocket = new SinglePortVirtualServerSocket(socket.getLocalPort());
      }
      catch (IOException e)
      {
         log.error("i/o error creating SinglePortVirtualServerSocket", e);
         e.printStackTrace();
      }

      Socket socket1 = null;
      Socket socket2 = null;

      try
      {
         socket1 = serverSocket.accept();
         socket2 = serverSocket.accept();
      }
      catch (IOException e)
      {
         log.error("i/o error creating virtual sockets", e);
         e.printStackTrace();
      }

      try
      {
         ObjectInputStream ois1 = new ObjectInputStream(socket1.getInputStream());
         ObjectInputStream ois2 = new ObjectInputStream(socket2.getInputStream());

         int b1 = ois1.read();
         int b2 = ois2.read();

         int i1 = ois1.readInt();
         int i2 = ois2.readInt();

         String s1 = ois1.readUTF();
         String s2 = ois2.readUTF();

         Integer ii1 = (Integer) ois1.readObject();
         Integer ii2 = (Integer) ois2.readObject();

         log.info("CLIENT: socket 1: asynchronous read byte:   " + b1);
         log.info("CLIENT: socket 2: asynchronous read byte:   " + b2);

         log.info("CLIENT: socket 1: asynchronous read int:    " + i1);
         log.info("CLIENT: socket 2: asynchronous read int:    " + i2);

         log.info("CLIENT: socket 1: asynchronous read String: " + s1);
         log.info("CLIENT: socket 2: asynchronous read String: " + s2);

         log.info("CLIENT: socket 1: asynchronous read Integer: " + ii1);
         log.info("CLIENT: socket 2: asynchronous read Integer: " + ii2);

      }
      catch (IOException e)
      {
         log.error("i/o error reading from virtual sockets", e);
         e.printStackTrace();
      } catch (ClassNotFoundException e)
      {
         log.error("class not found exception");
         e.printStackTrace();
      }

      try
      {
         serverSocket.close();
      }
      catch (IOException ignored)
      {
         log.error("i/o error closing server socket");
         ignored.printStackTrace();
      }

      log.info("Ending client callback test thread");
   }

}
