/*
 * Created on Jul 24, 2005
 *
 */
package org.jboss.test.remoting.transport.multiplex;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import org.jboss.logging.Logger;
import org.jboss.remoting.transport.multiplex.MultiPortVirtualServerSocket;
import org.jboss.remoting.transport.multiplex.VirtualSocket;

/**
 *
 * <p>
 * Copyright (c) 2005
 * <p>
 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 */
public class ServerTest
{
   private static final Logger log = Logger.getLogger(ServerTest.class);

   private Socket socket = null;
   private ServerSocket serverSocket = null;

   private ObjectOutputStream os = null;
   private ObjectInputStream  is = null;

   private boolean testOver = false;

   /**
    *
    */
   public ServerTest()
   {
   }


   protected void runTest()
   {
      log.info("Start server test");

      try
      {
         serverSocket = new MultiPortVirtualServerSocket(1999);
         log.info("ServerTest.main(): created MultiPortVirtualServerSocket");
      }
      catch (IOException e)
      {
         log.error("ServerTest.main(): cannot create MultiPortVirtualServerSocket");
         e.printStackTrace();
      }

      while (!testOver)
      {
         try
         {
            socket = serverSocket.accept();
            log.info("ServerTest.main(): created VirtualSocket");
            Thread thread = new ServerTestThread(socket);
            thread.start();
         }
         catch (SocketException ignored)
         {
            // it's because the ServerSocket was closed
         }
         catch (IOException e)
         {
            log.error("ServerTest.main(): cannot create VirtualSocket");
            e.printStackTrace();
         }
      }

      log.info("Ending server test (main thread)");
   }


   protected void testCallBacks(Socket socket)
   {
      log.info("Starting server callback tests");

      String hostName = socket.getInetAddress().getHostName();
      int port = socket.getPort();
      Socket socket1 = null;
      Socket socket2 = null;

      try
      {
         socket1 = new VirtualSocket(hostName, port);
         socket2 = new VirtualSocket(hostName, port);
      }
      catch (UnknownHostException e)
      {
         log.error("unknown host exception");
         e.printStackTrace();
      }
      catch (IOException e)
      {
         log.error("i/o exception");
         e.printStackTrace();
      }

      try
      {
         ObjectOutputStream oos1 = new ObjectOutputStream(socket1.getOutputStream());
         ObjectOutputStream oos2 = new ObjectOutputStream(socket2.getOutputStream());

         oos1.write(3);
         oos2.write(7);

         oos1.writeInt(11);
         oos2.writeInt(13);

         oos1.writeUTF("def");
         oos2.writeUTF("pqr");

         oos1.writeObject(new Integer(23));
         oos2.writeObject(new Integer(29));

         oos1.flush();
         oos2.flush();
      }
      catch (IOException e)
      {
         log.error("i/o error writing to virtual sockets", e);
         e.printStackTrace();
      }


      try
      {
         socket1.close();
         socket2.close();
      }
      catch (IOException ignored)
      {
         log.info("error closing sockets");
      }

      log.info("Ending server callback tests");
   }


   public static void main(String[] args)
   {
      new ServerTest().runTest();
   }



   class ServerTestThread extends Thread
   {
      private Socket socket;
      private ObjectInputStream  ois;
      private ObjectOutputStream oos;
      private DataInputStream  is;
//      private BufferedDataOutputStream os;


      public ServerTestThread(Socket socket)
      {
         this.socket = socket;

         try
         {
            ois = new ObjectInputStream(socket.getInputStream());
            oos = new ObjectOutputStream(socket.getOutputStream());
         }
         catch (IOException e)
         {
            log.error("ServerTest.ServerTestThread(): i/o error creating InputStream or OutputStream", e);
            e.printStackTrace();
         }
      }

      public void run()
      {
         log.info("Starting server test thread");

         testCallBacks(socket);

         try
         {
         	int i1 = ois.read();
            log.info("SERVER: synchronous read byte    " + i1);
         	oos.write(i1);

        	int i2 = ois.readInt();
            log.info("SERVER: synchronous read int:    " + i2);
         	oos.writeInt(i2);
         }
         catch (IOException e)
         {
            log.error("ServerTest.ServerTestThread.run(): i/o error", e);
            e.printStackTrace();
            return;
         }

         boolean stillReading = true;

         while (stillReading)
         {
            try
            {
               Object o = ois.readObject();

               if (o instanceof Boolean && ((Boolean) o).booleanValue() == false)
                  stillReading = false;

               log.info("SERVER: synchronous read object: " + o + " (" + o.getClass() + ")");
               oos.writeObject(o);
            }

            catch (IOException e)
            {
               log.error("ServerTest.ServerTestThread.run(): i/o error", e);
               e.printStackTrace();
               break;
            }
            catch (ClassNotFoundException e)
            {
               log.error("ServerTest.ServerTestThread.run(): class not found", e);
               e.printStackTrace();
               break;
            }
         }

         try
         {
            socket.close();
         }
         catch (IOException ignored)
         {
            log.info("i/o error closing socket");
         }

         testOver = true;

         try
         {
            serverSocket.close();
         }
         catch (IOException ignored)
         {
            log.info("i/o error closing server socket");
         }

         log.info("Ending server test thread");
      }
   }



}