/*
 * Created on Jul 24, 2005
 *
 */
package org.jboss.test.remoting.transport.multiplex;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import org.apache.log4j.Level;
import org.jboss.jrunit.extensions.ServerTestCase;
import org.jboss.logging.Logger;
import org.jboss.remoting.transport.multiplex.MultiPortVirtualServerSocket;

/**
 * <p/>
 * Copyright (c) 2005
 * <p/>
 *
 * @author <a href="mailto:r.sigal@computer.org">Ron Sigal</a>
 */
public class SimpleServerTest extends ServerTestCase
{
   private static final Logger log = Logger.getLogger(SimpleServerTest.class);

   private Socket socket = null;
   private ServerSocket serverSocket = null;

   private boolean testOver = false;

   public void setUp()
   {
      log.info("Start server test");

      try
      {
         serverSocket = new MultiPortVirtualServerSocket(1999);
         log.info("ServerTest.main(): created MultiPortVirtualServerSocket");
      }
      catch(IOException e)
      {
         log.error("ServerTest.main(): cannot create MultiPortVirtualServerSocket");
         e.printStackTrace();
      }

      new Thread(new Runnable() {

         public void run()
         {
            while(!testOver)
            {
               try
               {
                  socket = serverSocket.accept();
                  log.info("ServerTest.main(): created VirtualSocket");
                  Thread thread = new ServerTestThread(socket);
                  thread.start();
               }
               catch(SocketException ignored)
               {
                  // it's because the ServerSocket was closed
               }
               catch(IOException e)
               {
                  log.error("ServerTest.main(): cannot create VirtualSocket");
                  e.printStackTrace();
               }
            }

         }

      }).start();

   }

   public void tearDown()
   {
      testOver = true;

      try
      {
         serverSocket.close();
      }
      catch(IOException ignored)
      {
         log.info("i/o error closing server socket");
      }
      
   }


   public static void main(String[] args)
   {
      org.apache.log4j.BasicConfigurator.configure();
      org.apache.log4j.Category.getRoot().setLevel(Level.DEBUG);

      new SimpleServerTest().setUp();
   }


   class ServerTestThread extends Thread
   {
      private Socket socket;
      private ObjectInputStream ois;
      private ObjectOutputStream oos;
      private DataInputStream is;
//      private BufferedDataOutputStream os;


      public ServerTestThread(Socket socket)
      {
         this.socket = socket;

         try
         {
            ois = new ObjectInputStream(socket.getInputStream());
            oos = new ObjectOutputStream(socket.getOutputStream());
         }
         catch(IOException e)
         {
            log.error("ServerTest.ServerTestThread(): i/o error creating InputStream or OutputStream", e);
            e.printStackTrace();
         }
      }

      public void run()
      {
         log.info("Starting server test thread");

         boolean stillReading = true;

         while(stillReading)
         {
            try
            {
               Object o = ois.readObject();

               Integer i = (Integer) o;
               int val = i.intValue();
               val = val + 100;

               oos.writeObject(new Integer(val));
            }

            catch(IOException e)
            {
               log.error("ServerTest.ServerTestThread.run(): i/o error", e);
               e.printStackTrace();
               break;
            }
            catch(ClassNotFoundException e)
            {
               log.error("ServerTest.ServerTestThread.run(): class not found", e);
               e.printStackTrace();
               break;
            }
         }

         try
         {
            socket.close();
         }
         catch(IOException ignored)
         {
            log.info("i/o error closing socket");
         }

         log.info("Ending server test thread");
      }
   }


}