/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.transport.rmi;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import org.jboss.jrunit.extensions.ServerTestCase;
import org.jboss.logging.Logger;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.ServerInvocationHandler;
import org.jboss.remoting.marshal.Marshaller;
import org.jboss.remoting.marshal.MarshalFactory;
import org.jboss.remoting.marshal.serializable.SerializableMarshaller;
import org.jboss.remoting.transport.Connector;
import org.jboss.remoting.transport.rmi.RMIServerInvoker;
import org.jboss.test.remoting.TestUtil;
import org.jboss.test.remoting.transport.mock.MockServerInvocationHandler;


/**
 * This is the concrete test for invoker server.
 *
 * @author <a href="mailto:telrod@e2technologies.net">Tom Elrod</a>
 */
public class RMIInvokerServerTest extends ServerTestCase
{
   private int port = RMIServerInvoker.DEFAULT_REGISTRY_PORT - 1;
   private String transport = "rmi";
   private Connector connector;

   private static final Logger log = Logger.getLogger(RMIInvokerServerTest.class);

   public void init(Map metatdata) throws Exception
   {
      if(port < 0)
      {
         port = TestUtil.getRandomPort();
      }
      log.debug("port = " + port);

      connector = new Connector();
      InvokerLocator locator = new InvokerLocator(buildLocatorURI(metatdata));
      System.out.println("server locator: " + locator);
      connector.setInvokerLocator(locator.getLocatorURI());
      connector.start();
      connector.addInvocationHandler(getSubsystem(), getServerInvocationHandler());
   }

   private String buildLocatorURI(Map metadata)
   {
      if(metadata == null || metadata.size() == 0)
      {
         return transport + "://localhost:" + port;
      }
      else
      {
         StringBuffer uriBuffer = new StringBuffer(transport + "://localhost:" + port + "/?");

         Set keys = metadata.keySet();
         Iterator itr = keys.iterator();
         while(itr.hasNext())
         {
            String key = (String) itr.next();
            String value = (String) metadata.get(key);
            uriBuffer.append(key + "=" + value + "&");
         }
         return uriBuffer.substring(0, uriBuffer.length() - 1);
      }
   }

   protected String getSubsystem()
   {
      return "mock";
   }

   protected ServerInvocationHandler getServerInvocationHandler()
   {
      return new MockServerInvocationHandler();
   }

   public void setUp() throws Exception
   {
      Map metadata = new HashMap();
      metadata.put(RMIServerInvoker.REGISTRY_PORT_KEY, String.valueOf(port + 1));
      metadata.put(InvokerLocator.DATATYPE, SerializableMarshaller.DATATYPE);
      init(metadata);
   }

   public void tearDown() throws Exception
   {
      if(connector != null)
      {
         connector.stop();
         connector.destroy();
      }
   }

   public static void main(String[] args)
   {
      RMIInvokerServerTest server = new RMIInvokerServerTest();
      try
      {
         server.setUp();
         Thread.currentThread().sleep(6000000);
      }
      catch(Exception e)
      {
         e.printStackTrace();
      }
   }


}
