/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.remoting.transport.web;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.transport.http.HTTPServerInvoker;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public abstract class WebInvokerTestClient extends TestCase
{
   public abstract String getLocatorURI();

   public void testInvocation() throws Exception
   {
      Client remotingClient = null;

      try
      {
         InvokerLocator locator = new InvokerLocator(getLocatorURI());
         System.out.println("Calling remoting server with locator uri of: " + getLocatorURI());

         remotingClient = new Client(locator);

         Map metadata = new HashMap();
         metadata.put(Client.RAW, Boolean.TRUE);
         metadata.put("TYPE", "POST");

         Properties headerProps = new Properties();
         headerProps.put("Content-type", "application/soap+xml");

         metadata.put("HEADER", headerProps);

         Object response = null;

         // test with null return expected
         response = remotingClient.invoke(WebInvocationHandler.NULL_RETURN_PARAM, metadata);
         System.out.println("First response should be null and was: " + response);
         assertNull(response);

         response = remotingClient.invoke("Do something", metadata);
         System.out.println("Second response should be " + WebInvocationHandler.HTML_PAGE_RESPONSE + " and was: " + response);
         assertEquals(WebInvocationHandler.HTML_PAGE_RESPONSE, response);

         headerProps.put("Content-type", HTTPServerInvoker.BINARY);
         response = remotingClient.invoke(new ComplexObject(2, "foo", true), metadata);
         System.out.println("Third response should be " + WebInvocationHandler.OBJECT_RESPONSE_VALUE + " and was: " + response);
         assertEquals(WebInvocationHandler.OBJECT_RESPONSE_VALUE, response);

         response = remotingClient.invoke(WebInvocationHandler.STRING_RETURN_PARAM, metadata);
         System.out.println("Fourth response should be " + WebInvocationHandler.RESPONSE_VALUE + " and was: " + response);
         assertEquals(WebInvocationHandler.RESPONSE_VALUE, response);

         try
         {
            response = remotingClient.invoke(WebInvocationHandler.THROW_EXCEPTION_PARAM, metadata);
            System.out.println("Should have thrown exception, but did not.");
            assertTrue("Should have received an exception.", false);
         }
         catch(Throwable throwable)
         {
            System.out.println("Received exception as expected.");
            assertTrue("Received exception as expected", true);
         }

         remotingClient.invokeOneway("Do something", metadata, true);

         remotingClient.invokeOneway("Do something", metadata, false);
      }
      catch(Throwable throwable)
      {
         throw new Exception(throwable);
      }
      finally
      {
         if(remotingClient != null)
         {
            remotingClient.disconnect();
         }
      }


   }
}